@@ -1,9 +1,8 @@
 /* GDA library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998-2002 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
- *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -21,16 +20,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <string.h>
 #include <glib/garray.h>
 #include <libgda/gda-data-model-array.h>
-#include <glib/gi18n-lib.h>
-#include <libgda/gda-row.h>
-#include <libgda/gda-data-model.h>
-#include <libgda/gda-data-model-extra.h>
-#include <libgda/gda-util.h>
 
-#define PARENT_TYPE GDA_TYPE_DATA_MODEL_ROW
+#define PARENT_TYPE GDA_TYPE_DATA_MODEL
 
 struct _GdaDataModelArrayPrivate {
 	/* number of columns in each row */
@@ -52,52 +45,47 @@ static GObjectClass *parent_class = NULL;
  */
 
 static gint
-gda_data_model_array_get_n_rows (GdaDataModelRow *model)
+gda_data_model_array_get_n_rows (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), -1);
 	return GDA_DATA_MODEL_ARRAY (model)->priv->rows->len;
 }
 
 static gint
-gda_data_model_array_get_n_columns (GdaDataModelRow *model)
+gda_data_model_array_get_n_columns (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), -1);
 	return GDA_DATA_MODEL_ARRAY (model)->priv->number_of_columns;
 }
 
-static GdaRow *
-gda_data_model_array_get_row (GdaDataModelRow *model, gint row, GError **error)
+static GdaFieldAttributes *
+gda_data_model_array_describe_column (GdaDataModel *model, gint col)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), NULL);
 
-	if (row >= GDA_DATA_MODEL_ARRAY (model)->priv->rows->len) {
-		g_set_error (error, 0, 0,
-			     _("Row %d out of range 0 - %d"), row,
-			     GDA_DATA_MODEL_ARRAY (model)->priv->rows->len- 1);
-		return NULL;
-	}
+	return NULL;
+}
 
-	return GDA_ROW (g_ptr_array_index (GDA_DATA_MODEL_ARRAY (model)->priv->rows, row));
+static const GdaRow *
+gda_data_model_array_get_row (GdaDataModel *model, gint row)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), NULL);
+
+	if (row >= GDA_DATA_MODEL_ARRAY (model)->priv->rows->len)
+		return NULL;
+
+	return (const GdaRow *) g_ptr_array_index (GDA_DATA_MODEL_ARRAY (model)->priv->rows, row);
 }
 
 static const GdaValue *
-gda_data_model_array_get_value_at (GdaDataModelRow *model, gint col, gint row)
+gda_data_model_array_get_value_at (GdaDataModel *model, gint col, gint row)
 {
 	GdaRow *fields;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), NULL);
 
-	if (row >= GDA_DATA_MODEL_ARRAY (model)->priv->rows->len) {
-		g_warning (_("Row %d out of range 0 - %d"), row, 
-			   GDA_DATA_MODEL_ARRAY (model)->priv->rows->len);
+	if (row >= GDA_DATA_MODEL_ARRAY (model)->priv->rows->len)
 		return NULL;
-	}
-
-	if (col >= GDA_DATA_MODEL_ARRAY (model)->priv->number_of_columns) {
-		g_warning (_("Column out %d of range 0 - %d"), col, 
-			   GDA_DATA_MODEL_ARRAY (model)->priv->number_of_columns);
-		return NULL;
-	}
 
 	fields = g_ptr_array_index (GDA_DATA_MODEL_ARRAY (model)->priv->rows, row);
 	if (fields != NULL) {
@@ -111,14 +99,14 @@ gda_data_model_array_get_value_at (GdaDataModelRow *model, gint col, gint row)
 }
 
 static gboolean
-gda_data_model_array_is_updatable (GdaDataModelRow *model)
+gda_data_model_array_is_updatable (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
 	return TRUE;
 }
 
-static GdaRow *
-gda_data_model_array_append_values (GdaDataModelRow *model, const GList *values, GError **error)
+static const GdaRow *
+gda_data_model_array_append_row (GdaDataModel *model, const GList *values)
 {
 	gint len;
 	GdaRow *row = NULL;
@@ -127,97 +115,94 @@ gda_data_model_array_append_values (GdaDataModelRow *model, const GList *values,
 	g_return_val_if_fail (values != NULL, NULL);
 
 	len = g_list_length ((GList *) values);
-	if (len != GDA_DATA_MODEL_ARRAY (model)->priv->number_of_columns) {
-		g_set_error (error, 0, GDA_DATA_MODEL_VALUES_LIST_ERROR,
-			     _("Wrong number of values in values list"));
+	if (len != GDA_DATA_MODEL_ARRAY (model)->priv->number_of_columns)
 		return NULL;
+
+	row = gda_row_new_from_list (model, values);
+	if (row) {
+		g_ptr_array_add (GDA_DATA_MODEL_ARRAY (model)->priv->rows, row);
+		gda_row_set_number (row, GDA_DATA_MODEL_ARRAY (model)->priv->rows->len - 1);
+		gda_data_model_row_inserted (model, GDA_DATA_MODEL_ARRAY (model)->priv->rows->len - 1);
 	}
 
-	row = gda_row_new_from_list (GDA_DATA_MODEL (model), values);
-	if (!GDA_DATA_MODEL_ROW_CLASS (G_OBJECT_GET_CLASS (model))->append_row (model, row, error)) {
-		g_object_unref (row);
-		return NULL;
-	}
-
-	g_object_unref (row);
-	return (GdaRow *) row;
+	return (const GdaRow *) row;
 }
 
 static gboolean
-gda_data_model_array_append_row (GdaDataModelRow *model, GdaRow *row, GError **error)
+gda_data_model_array_remove_row (GdaDataModel *model, const GdaRow *row)
 {
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
-	g_return_val_if_fail (row != NULL, FALSE);
-
-	g_ptr_array_add (GDA_DATA_MODEL_ARRAY (model)->priv->rows, (GdaRow *) row);
-	g_object_ref (row);
-	gda_row_set_number ((GdaRow *) row, GDA_DATA_MODEL_ARRAY (model)->priv->rows->len - 1);
-	gda_data_model_row_inserted ((GdaDataModel *) model, GDA_DATA_MODEL_ARRAY (model)->priv->rows->len - 1);
-
-	return TRUE;
-}
-
-static gboolean
-gda_data_model_array_update_row (GdaDataModelRow *model, GdaRow *row, GError **error)
-{
-        gint i;
-        GdaDataModelArrayPrivate *priv;
-	gint num;
-
-        g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
-        g_return_val_if_fail (row != NULL, FALSE);
-
-	num = gda_row_get_number (row);
-        priv = GDA_DATA_MODEL_ARRAY (model)->priv;
-
-        for (i = 0; i < priv->rows->len; i++) {
-                if (gda_row_get_number (priv->rows->pdata[i]) == num) {
-			GdaRow *tmp;
-
-			if (row == priv->rows->pdata[i]) {
-				gda_data_model_row_updated ((GdaDataModel *) model, i);
-				return TRUE;
-			}
-
-			tmp = gda_row_copy (row);
-			g_object_unref (priv->rows->pdata[i]);
-			priv->rows->pdata[i] = tmp;
-			gda_data_model_row_updated ((GdaDataModel *) model, i);
-
-                        return TRUE;
-                }
-        }
-
-	g_set_error (error, 0, GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
-		     _("Row not found in data model"));
-        return FALSE;
-}
-
-static gboolean
-gda_data_model_array_remove_row (GdaDataModelRow *model, GdaRow *row, GError **error)
-{
-	gint i, rownum;
+	gint      i, rownum;
 	
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
 	g_return_val_if_fail (row != NULL, FALSE);
 
+	/* remove entry from the array */
 	if (g_ptr_array_remove (GDA_DATA_MODEL_ARRAY (model)->priv->rows, (gpointer) row)) {
 		/* renumber following rows */
 		rownum = gda_row_get_number ((GdaRow *) row);
 		for (i = (rownum + 1); i < GDA_DATA_MODEL_ARRAY (model)->priv->rows->len; i++)
-			gda_row_set_number ((GdaRow *) gda_data_model_array_get_row (model, i, error), (i-1));
+			gda_row_set_number ((GdaRow *) gda_data_model_get_row (GDA_DATA_MODEL (model), i), (i-1));
 
 		/* tag the row as being removed */
 		gda_row_set_id ((GdaRow *) row, "R");
 		gda_row_set_number ((GdaRow *) row, -1);
 
-		gda_data_model_row_removed ((GdaDataModel *) model, rownum);
-		g_object_unref (row);
+		/* emit updated signals */
+		gda_data_model_row_removed (model, rownum);
+
 		return TRUE;
 	}
 
-	g_set_error (error, 0, GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
-		     _("Row not found in data model"));
+	return FALSE;
+}
+
+static gboolean
+gda_data_model_array_update_row (GdaDataModel *model, const GdaRow *row)
+{
+	gint i;
+	GdaDataModelArrayPrivate *priv;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
+	g_return_val_if_fail (row != NULL, FALSE);
+
+	priv = GDA_DATA_MODEL_ARRAY (model)->priv;
+
+	for (i = 0; i < priv->rows->len; i++) {
+		if (priv->rows->pdata[i] == row) {
+			gda_row_free (priv->rows->pdata[i]);
+			priv->rows->pdata[i] = gda_row_copy ((GdaRow *) row);
+			gda_data_model_row_updated (model, i);
+
+			return TRUE;
+		}
+	}
+
+	return FALSE; /* row not found in this data model */
+}
+
+static gboolean
+gda_data_model_array_append_column (GdaDataModel *model, const GdaFieldAttributes *attrs)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
+	g_return_val_if_fail (attrs != NULL, FALSE);
+
+	return FALSE;
+}
+
+static gboolean
+gda_data_model_array_update_column (GdaDataModel *model, gint col, const GdaFieldAttributes *attrs)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
+	g_return_val_if_fail (attrs != NULL, FALSE);
+
+	return FALSE;
+}
+
+static gboolean
+gda_data_model_array_remove_column (GdaDataModel *model, gint col)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
+
 	return FALSE;
 }
 
@@ -225,20 +210,23 @@ static void
 gda_data_model_array_class_init (GdaDataModelArrayClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelRowClass *model_class = GDA_DATA_MODEL_ROW_CLASS (klass);
+	GdaDataModelClass *model_class = GDA_DATA_MODEL_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_data_model_array_finalize;
 	model_class->get_n_rows = gda_data_model_array_get_n_rows;
 	model_class->get_n_columns = gda_data_model_array_get_n_columns;
+	model_class->describe_column = gda_data_model_array_describe_column;
 	model_class->get_row = gda_data_model_array_get_row;
 	model_class->get_value_at = gda_data_model_array_get_value_at;
 	model_class->is_updatable = gda_data_model_array_is_updatable;
-	model_class->append_values = gda_data_model_array_append_values;
 	model_class->append_row = gda_data_model_array_append_row;
-	model_class->update_row = gda_data_model_array_update_row;
 	model_class->remove_row = gda_data_model_array_remove_row;
+	model_class->update_row = gda_data_model_array_update_row;
+	model_class->append_column = gda_data_model_array_append_column;
+	model_class->update_column = gda_data_model_array_update_column;
+	model_class->remove_column = gda_data_model_array_remove_column;
 }
 
 static void
@@ -287,7 +275,9 @@ gda_data_model_array_get_type (void)
 			0,
 			(GInstanceInitFunc) gda_data_model_array_init
 		};
-		type = g_type_register_static (PARENT_TYPE, "GdaDataModelArray", &info, 0);
+		type = g_type_register_static (PARENT_TYPE,
+					       "GdaDataModelArray",
+					       &info, 0);
 	}
 	return type;
 }
@@ -308,54 +298,6 @@ gda_data_model_array_new (gint cols)
 	return model;
 }
 
-/**
- * gda_data_model_array_copy_model
- * @src: a #GdaDataModel to copy data from
- * @error: a place to store errors, or %NULL
- *
- * Makes a copy of @src into a new #GdaDataModelArray object
- *
- * Returns: a new data model, or %NULL if an error occured
- */
-GdaDataModel *
-gda_data_model_array_copy_model (GdaDataModel *src, GError **error)
-{
-	GdaDataModel *model;
-	gint nbfields, i;
-
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (src), NULL);
-
-	nbfields = gda_data_model_get_n_columns (src);
-	model = gda_data_model_array_new (nbfields);
-
-	gda_object_set_name (GDA_OBJECT (model), gda_object_get_name (GDA_OBJECT (src)));
-	gda_object_set_description (GDA_OBJECT (model), gda_object_get_description (GDA_OBJECT (src)));
-	for (i = 0; i < nbfields; i++) {
-		GdaColumn *copycol, *srccol;
-
-		srccol = gda_data_model_describe_column (src, i);
-		copycol = gda_data_model_describe_column (model, i);
-
-		gda_column_set_title (copycol, gda_column_get_title (srccol));
-		gda_column_set_defined_size (copycol, gda_column_get_defined_size (srccol));
-		gda_column_set_name (copycol, gda_column_get_name (srccol));
-		gda_column_set_caption (copycol, gda_column_get_caption (srccol));
-		gda_column_set_scale (copycol, gda_column_get_scale (srccol));
-		gda_column_set_dbms_type (copycol, gda_column_get_dbms_type (srccol));
-		gda_column_set_gda_type (copycol, gda_column_get_gda_type (srccol));
-		gda_column_set_position (copycol, gda_column_get_position (srccol));
-	}
-
-	if (! gda_data_model_import_from_model (model, src, NULL, error)) {
-		g_object_unref (model);
-		model = NULL;
-	}
-	else
-		gda_data_model_dump (model, stdout);
-
-	return model;
-}
-
 /**
  * gda_data_model_array_set_n_columns
  * @model: the #GdaDataModelArray.
@@ -387,7 +329,7 @@ gda_data_model_array_clear (GdaDataModelArray *model)
 		GdaRow *row = (GdaRow *) g_ptr_array_index (model->priv->rows, 0);
 
 		if (row != NULL)
-			g_object_unref (row);
+			gda_row_free (row);
 		g_ptr_array_remove_index (model->priv->rows, 0);
 	}
 }
