@@ -1,10 +1,9 @@
 /* GDA library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998-2005 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
  *	Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
- *      Vivien Malerba <malerba@gnome-db.org>
  *	Bas Driessen <bas.driessen@xobas.com>
  *
  * This Library is free software; you can redistribute it and/or
@@ -24,11 +23,9 @@
  */
 
 #include <glib/ghash.h>
-#include <glib/gi18n-lib.h>
 #include <libgda/gda-data-model-hash.h>
-#include <libgda/gda-data-model-extra.h>
 
-#define PARENT_TYPE GDA_TYPE_DATA_MODEL_ROW
+#define PARENT_TYPE GDA_TYPE_DATA_MODEL
 
 struct _GdaDataModelHashPrivate {
 	/* number of columns in each row */
@@ -54,7 +51,7 @@ static GObjectClass *parent_class = NULL;
  */
 
 static gint
-gda_data_model_hash_get_n_rows (GdaDataModelRow *model)
+gda_data_model_hash_get_n_rows (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), -1);
 
@@ -65,118 +62,91 @@ gda_data_model_hash_get_n_rows (GdaDataModelRow *model)
 }
 
 static gint
-gda_data_model_hash_get_n_columns (GdaDataModelRow *model)
+gda_data_model_hash_get_n_columns (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), -1);
 	return GDA_DATA_MODEL_HASH (model)->priv->number_of_columns;
 }
 
-static GdaRow *
-gda_data_model_hash_get_row (GdaDataModelRow *model, gint row, GError **error)
+static GdaFieldAttributes *
+gda_data_model_hash_describe_column (GdaDataModel *model, gint col)
 {
-	gint hash_entry;
-	GdaRow *gdarow = NULL;
-
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
 
-	/* get row according mapping */
-	if (row < GDA_DATA_MODEL_HASH (model)->priv->row_map->len) {
-		hash_entry = g_array_index (GDA_DATA_MODEL_HASH (model)->priv->row_map, gint, row);
-		
-		gdarow = (GdaRow *) g_hash_table_lookup (GDA_DATA_MODEL_HASH (model)->priv->rows,
-							 GINT_TO_POINTER (hash_entry));
-	}
-	if (!gdarow)
-		g_set_error (error, 0, 0, _("Row %d not found in data model"), row);
-
-	return gdarow;
-}
-
-static const GValue *
-gda_data_model_hash_get_value_at (GdaDataModelRow *model, gint col, gint row)
-{
-	const GdaRow *fields;
-
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
-
-	fields = gda_data_model_hash_get_row (model, row, NULL);
-	if (fields == NULL)
-		return NULL;
-
-	if (col >= GDA_DATA_MODEL_HASH (model)->priv->number_of_columns) {
-		g_warning (_("Column out %d of range 0 - %d"), col, 
-			   GDA_DATA_MODEL_HASH (model)->priv->number_of_columns);
-		return NULL;
-	}
-
-	return (const GValue *) gda_row_get_value ((GdaRow *) fields, col);
+	return NULL;
 }
 
 static gboolean
-gda_data_model_hash_is_updatable (GdaDataModelRow *model)
+gda_data_model_hash_is_updatable (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), FALSE);
 	return TRUE;
 }
 
-static gboolean
-gda_data_model_hash_append_row (GdaDataModelRow *model, GdaRow *row, GError **error)
+static const GdaRow *
+gda_data_model_hash_append_row (GdaDataModel *model, const GList *values)
 {
+	GdaRow *row;
 	gint cols, rownum_hash_new, rownum_array_new;
 
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), FALSE);
-	g_return_val_if_fail (row != NULL, FALSE);
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
+	g_return_val_if_fail (values != NULL, NULL);
 
-	cols = gda_row_get_length (row);
-	if (cols != GDA_DATA_MODEL_HASH (model)->priv->number_of_columns) {
-		g_set_error (error, 0, GDA_DATA_MODEL_VALUES_LIST_ERROR,
-			     _("Wrong number of values in values list"));
-		return FALSE;
-	}
+	cols = g_list_length ((GList *) values);
+	if (cols != GDA_DATA_MODEL_HASH (model)->priv->number_of_columns)
+		return NULL;
+
+	/* create the GdaRow to add */
+	row = gda_row_new_from_list (model, values);
 
 	/* get the new hash and array row number */
 	rownum_hash_new = GDA_DATA_MODEL_HASH (model)->priv->number_of_hash_table_rows;
 	rownum_array_new = GDA_DATA_MODEL_HASH (model)->priv->row_map->len;
 
-	gda_data_model_hash_insert_row (
-		GDA_DATA_MODEL_HASH (model),
-		rownum_hash_new,
-		row);
+	if (row) {
+		gda_data_model_hash_insert_row (
+			GDA_DATA_MODEL_HASH (model),
+			rownum_hash_new,
+			row);
+		gda_row_set_number (row, rownum_array_new);
 
-	gda_row_set_number (row, rownum_array_new);
+		/* append row number in row mapping array */
+		g_array_append_val (GDA_DATA_MODEL_HASH (model)->priv->row_map, rownum_hash_new);
 
-	/* Append row number in row mapping array */
-	g_array_append_val (GDA_DATA_MODEL_HASH (model)->priv->row_map, rownum_hash_new);
+		/* emit update signals */
+		gda_data_model_row_inserted (GDA_DATA_MODEL (model), rownum_array_new);
+		gda_data_model_changed (GDA_DATA_MODEL (model));
 
-	/* emit update signals */
-	gda_data_model_row_inserted (GDA_DATA_MODEL (model), rownum_array_new);
+		/* increment the number of entries in the hash table */
+		GDA_DATA_MODEL_HASH (model)->priv->number_of_hash_table_rows++;
+	}
 
-	/* increment the number of entries in the hash table */
-	GDA_DATA_MODEL_HASH (model)->priv->number_of_hash_table_rows++;
-
-	return TRUE;
+	return row;
 }
 
 static gboolean
-gda_data_model_hash_remove_row (GdaDataModelRow *model, GdaRow *row, GError **error)
+gda_data_model_hash_remove_row (GdaDataModel *model, const GdaRow *row)
 {
 	gint i, cols, rownum;
+	GdaValue *value;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), FALSE);
 	g_return_val_if_fail (row != NULL, FALSE);
 
 	cols = GDA_DATA_MODEL_HASH (model)->priv->number_of_columns;
 
-	for (i=0; i<cols; i++)
-		gda_row_set_value (row, i, NULL);
+	for (i=0; i<cols; i++) {
+		value = (GdaValue *) gda_row_get_value ((GdaRow *) row, i);
+		gda_value_set_string (value, "NULL");
+	}
 
 	rownum = gda_row_get_number ((GdaRow *) row);
 
 	/* renumber following rows */
-	for (i=(rownum+1); i<GDA_DATA_MODEL_HASH (model)->priv->row_map->len; i++) 
-		gda_row_set_number ((GdaRow *) gda_data_model_hash_get_row (model, i, error), (i-1));
+	for (i=(rownum+1); i<GDA_DATA_MODEL_HASH (model)->priv->row_map->len; i++)
+		gda_row_set_number ((GdaRow *) gda_data_model_get_row (GDA_DATA_MODEL (model), i), (i-1));
 
-	/* tag the row as being removed */	
+	/* tag the row as being removed */
 	gda_row_set_id ((GdaRow *) row, "R");
 	gda_row_set_number ((GdaRow *) row, -1);
 
@@ -185,59 +155,33 @@ gda_data_model_hash_remove_row (GdaDataModelRow *model, GdaRow *row, GError **er
 
 	/* emit updated signals */
 	gda_data_model_row_removed (GDA_DATA_MODEL (model), gda_row_get_number ((GdaRow *) row));
-	
+	gda_data_model_changed (GDA_DATA_MODEL (model));
+
 	return TRUE;
 }
 
-static GdaRow *
-gda_data_model_hash_append_values (GdaDataModelRow *model, const GList *values, GError **error)
-{
-	gint len;
-	GdaRow *row = NULL;
-
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
-	g_return_val_if_fail (values != NULL, NULL);
-
-	len = g_list_length ((GList *) values);
-	if (len != GDA_DATA_MODEL_HASH (model)->priv->number_of_columns) {
-		g_set_error (error, 0, GDA_DATA_MODEL_VALUES_LIST_ERROR,
-			     _("Wrong number of values in values list"));
-		return NULL;
-	}
-
-	row = gda_row_new_from_list (GDA_DATA_MODEL (model), values);
-	if (!GDA_DATA_MODEL_ROW_CLASS (G_OBJECT_GET_CLASS (model))->append_row (model, row, error)) {
-		g_object_unref (row);
-		return NULL;
-	}
-
-	g_object_unref (row);
-	return (GdaRow *) row;
-}
-
 static void
 free_hash (gpointer value)
 {
-	g_object_unref (value);
+	gda_row_free ((GdaRow *) value);
 }
 
 static void
 gda_data_model_hash_class_init (GdaDataModelHashClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelRowClass *model_class = GDA_DATA_MODEL_ROW_CLASS (klass);
+	GdaDataModelClass *model_class = GDA_DATA_MODEL_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_data_model_hash_finalize;
 	model_class->get_n_rows = gda_data_model_hash_get_n_rows;
 	model_class->get_n_columns = gda_data_model_hash_get_n_columns;
+	model_class->describe_column = gda_data_model_hash_describe_column;
 	model_class->get_row = gda_data_model_hash_get_row;
 	model_class->get_value_at = gda_data_model_hash_get_value_at;
 	model_class->is_updatable = gda_data_model_hash_is_updatable;
-	model_class->append_values = gda_data_model_hash_append_values;
 	model_class->append_row = gda_data_model_hash_append_row;
-	model_class->update_row = NULL;
 	model_class->remove_row = gda_data_model_hash_remove_row;
 }
 
@@ -323,6 +267,29 @@ gda_data_model_hash_new (gint cols)
 	return model;
 }
 
+/**
+ * gda_data_model_hash_get_value_at
+ * @model: the #GdaDataModelHash to retrieve the value from.
+ * @col: column number (starting from 0).
+ * @row: row number (starting from 0).
+ *
+ * Retrieves the value at a specified column and row.
+ * Returns: a pointer to a #GdaValue.
+ */
+const GdaValue *
+gda_data_model_hash_get_value_at (GdaDataModel *model, gint col, gint row)
+{
+	const GdaRow *fields;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
+
+	fields = gda_data_model_hash_get_row (model, row);
+	if (fields == NULL)
+		return NULL;
+
+	return (const GdaValue *) gda_row_get_value ((GdaRow *) fields, col);
+}
+
 /**
  * gda_data_model_hash_insert_row
  * @model: the #GdaDataModelHash which is gonna hold the row.
@@ -352,8 +319,31 @@ gda_data_model_hash_insert_row (GdaDataModelHash *model,
 
 	g_hash_table_insert (model->priv->rows,
 			     GINT_TO_POINTER (rownum), row);
-	g_object_ref (row);
 	gda_data_model_row_inserted (GDA_DATA_MODEL (model), rownum);
+	gda_data_model_changed (GDA_DATA_MODEL (model));
+}
+
+/**
+ * gda_data_model_hash_get_row
+ * @model: the GdaDataModelHash
+ * @row: row number
+ *
+ * Retrieves a row from the underlying hash table.
+ *
+ * Returns: a #GdaRow or NULL if the requested row is not in the hash table.
+ */
+const GdaRow *
+gda_data_model_hash_get_row (GdaDataModel *model, gint row)
+{
+	gint hash_entry;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), NULL);
+
+	/* get row according mapping */
+	hash_entry = g_array_index (GDA_DATA_MODEL_HASH (model)->priv->row_map, gint, row);
+
+	return (const GdaRow *) g_hash_table_lookup (GDA_DATA_MODEL_HASH (model)->priv->rows,
+						     GINT_TO_POINTER (hash_entry));
 }
 
 /**
@@ -362,9 +352,7 @@ gda_data_model_hash_insert_row (GdaDataModelHash *model,
  * @cols: the number of columns for rows inserted in @model.
  *
  * Sets the number of columns for rows inserted in this model.
- * @cols must be greater than or equal to 0. 
- *
- * Also clears @model's contents.
+ * @cols must be greater than or equal to 0.
  *
  * This function calls #gda_data_model_hash_clear to free the
  * existing rows if any.
@@ -375,8 +363,8 @@ gda_data_model_hash_set_n_columns (GdaDataModelHash *model, gint cols)
 	g_return_if_fail (GDA_IS_DATA_MODEL_HASH (model));
 	g_return_if_fail (cols >= 0);
 
-	gda_data_model_hash_clear (model);
 	model->priv->number_of_columns = cols;
+	gda_data_model_hash_clear (model);
 }
 
 /**
@@ -411,10 +399,8 @@ gda_data_model_hash_clear (GdaDataModelHash *model)
 	/* create row mapping array */
 	model->priv->row_map = g_array_new (FALSE, FALSE, sizeof (gint));
 
-        /* Initialize row mapping array */
-        for (i = 0; i < model->priv->number_of_hash_table_rows; i++)
-                g_array_append_val (model->priv->row_map, i);
-
-	/* FIXME: signal changes */
+	/* Initialize row mapping array */
+	for (i = 0; i < model->priv->number_of_hash_table_rows; i++)
+		g_array_append_val (model->priv->row_map, i);
 }
 
