@@ -1,9 +1,8 @@
 /* GDA common library
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998-2002 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
- *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -27,7 +26,6 @@
 #include <libgda/gda-intl.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-select.h>
-#include <libgda/gda-data-model.h>
 #include <libgda/gda-value.h>
 #include <string.h>
 
@@ -63,8 +61,8 @@ data_model_changed_cb (GdaDataModel *model, gpointer user_data)
  * GdaSelect class implementation
  */
 
-static GdaColumn *
-gda_select_describe_column (GdaDataModelBase *model, gint col)
+static GdaFieldAttributes *
+gda_select_describe_column (GdaDataModel *model, gint col)
 {
 	GList *l;
 	GdaSelect *sel = (GdaSelect *) model;
@@ -76,11 +74,11 @@ gda_select_describe_column (GdaDataModelBase *model, gint col)
 	if (!l)
 		return NULL;
 
-	return gda_column_copy ((GdaColumn *) l->data);
+	return gda_field_attributes_copy ((GdaFieldAttributes *) l->data);
 }
 
 static const GdaRow *
-gda_select_get_row (GdaDataModelBase *model, gint row)
+gda_select_get_row (GdaDataModel *model, gint row)
 {
 	GdaSelect *sel = (GdaSelect *) model;
 
@@ -89,11 +87,11 @@ gda_select_get_row (GdaDataModelBase *model, gint row)
 	/* FIXME: identify this row, so that it can be updated and changes
 	   proxied to the source_model */
 
-	return GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_row (model, row);
+	return GDA_DATA_MODEL_CLASS (parent_class)->get_row (model, row);
 }
 
 static gboolean
-gda_select_is_updatable (GdaDataModelBase *model)
+gda_select_is_updatable (GdaDataModel *model)
 {
 	GdaSelect *sel = (GdaSelect *) model;
 
@@ -103,7 +101,7 @@ gda_select_is_updatable (GdaDataModelBase *model)
 }
 
 static const GdaRow *
-gda_select_append_values (GdaDataModelBase *model, const GList *values)
+gda_select_append_row (GdaDataModel *model, const GList *values)
 {
 	GdaSelect *sel = (GdaSelect *) model;
 
@@ -116,7 +114,7 @@ static void
 gda_select_class_init (GdaSelectClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelBaseClass *model_class = GDA_DATA_MODEL_BASE_CLASS (klass);
+	GdaDataModelClass *model_class = GDA_DATA_MODEL_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -126,7 +124,7 @@ gda_select_class_init (GdaSelectClass *klass)
 	model_class->get_row = gda_select_get_row;
 	/* we use the get_value_at of the base class */
 	model_class->is_updatable = gda_select_is_updatable;
-	model_class->append_values = gda_select_append_values;
+	model_class->append_row = gda_select_append_row;
 }
 
 static void
@@ -161,7 +159,7 @@ gda_select_finalize (GObject *object)
 
 	/* free memory */
 	if (sel->priv->field_descriptions) {
-		g_list_foreach (sel->priv->field_descriptions, (GFunc) gda_column_free, NULL);
+		g_list_foreach (sel->priv->field_descriptions, (GFunc) gda_field_attributes_free, NULL);
 		g_list_free (sel->priv->field_descriptions);
 		sel->priv->field_descriptions = NULL;
 	}
@@ -323,9 +321,9 @@ populate_from_single_table (GdaSelect *sel, const gchar *table_name, GList *sql_
 		GList *value_list = NULL;
 
 		for (c = 0; c < cols; c++) {
-			GdaColumn *column;
+			GdaFieldAttributes *fa;
 
-			column = gda_data_model_describe_column (table, c);
+			fa = gda_data_model_describe_column (table, c);
 
 			if (all_fields) {
 				value_list = g_list_append (
@@ -333,25 +331,25 @@ populate_from_single_table (GdaSelect *sel, const gchar *table_name, GList *sql_
 					gda_value_copy (gda_data_model_get_value_at (table, c, r)));
 				if (r == 0) {
 					sel->priv->field_descriptions = g_list_append (
-						sel->priv->field_descriptions, column);
+						sel->priv->field_descriptions, fa);
 				}
 			} else {
 				for (l = sql_fields; l != NULL; l = l->next) {
-					if (!strcmp ((const char *) l->data, gda_column_get_name (column))) {
+					if (!strcmp ((const char *) l->data, fa->name)) {
 						value_list = g_list_append (
 							value_list,
 							gda_value_copy (
 								gda_data_model_get_value_at (table, c, r)));
 						if (r == 0) {
 							sel->priv->field_descriptions = g_list_append (
-								sel->priv->field_descriptions, column);
+								sel->priv->field_descriptions, fa);
 						}
 					}
 				}
 			}
 		}
 
-		GDA_DATA_MODEL_BASE_CLASS (parent_class)->append_values (GDA_DATA_MODEL_BASE (sel), value_list);
+		GDA_DATA_MODEL_CLASS (parent_class)->append_row (GDA_DATA_MODEL (sel), value_list);
 
 		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
 		g_list_free (value_list);
@@ -386,7 +384,7 @@ gda_select_run (GdaSelect *sel)
 		return sel->priv->run_result;
 
 	gda_data_model_array_clear (GDA_DATA_MODEL_ARRAY (sel));
-	g_list_foreach (sel->priv->field_descriptions, (GFunc) gda_column_free, NULL);
+	g_list_foreach (sel->priv->field_descriptions, (GFunc) gda_field_attributes_free, NULL);
 	g_list_free (sel->priv->field_descriptions);
 
 	/* parse the SQL command */
