@@ -1,9 +1,8 @@
 /* GDA common library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998-2002 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
- *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -26,9 +25,8 @@
 
 #include <glib/ghash.h>
 #include <glib/glist.h>
-#include "gda-parameter.h"
-#include "gda-row.h"
-#include "gda-connection.h"
+#include <libgda/gda-parameter.h>
+#include <libgda/gda-row.h>
 
 G_BEGIN_DECLS
 
@@ -37,12 +35,6 @@ GdaValueType gda_type_from_string (const gchar *str);
 
 GList       *gda_string_hash_to_list (GHashTable *hash_table);
 
-/* 
- * SQL escaping
- */
-gchar *gda_default_escape_chars (const gchar *string);
-gchar *gda_default_unescape_chars (const gchar *string);
-
 /*
  * SQL parsing utilities
  */
@@ -52,28 +44,10 @@ gchar *gda_sql_replace_placeholders (const gchar *sql, GdaParameterList *params)
 /*
  * File management utility functions
  */
+
 gchar    *gda_file_load (const gchar *filename);
 gboolean  gda_file_save (const gchar *filename, const gchar *buffer, gint len);
 
-/*
- * utilities dealing with storing and retreiving GdaDictField's attributes
- * which is a list of comma separated keywords
- */
-gchar *utility_table_field_attrs_stringify (guint attributes);
-guint utility_table_field_attrs_parse     (const gchar *str);
-
-/*
- * XML Id encoding and decoding
- */
-gchar *utility_build_encoded_id (const gchar *prefix, const gchar *id);
-gchar *utility_build_decoded_id (const gchar *prefix, const gchar *id);
-
-/*
- * Check model columns
- */
-gboolean utility_check_data_model (GdaDataModel *model, gint nbcols, ...);
-
-
 G_END_DECLS
 
 #endif
