@@ -1,9 +1,8 @@
 /* GDA FreeTDS Provider
- * Copyright (C) 2002 - 2006 The GNOME Foundation
+ * Copyright (C) 2002 The GNOME Foundation
  *
  * AUTHORS:
  *         Holger Thon <holger.thon@gnome-db.org>
- *         Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,17 +23,13 @@
 #endif
 
 #include <libgda/gda-data-model-array.h>
-#include <libgda/gda-data-model-private.h>
-#include <glib/gi18n-lib.h>
+#include <libgda/gda-intl.h>
 #include <libgda/gda-log.h>
 #include <glib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <tds.h>
 #include "gda-freetds.h"
-#include "gda-freetds-defs.h"
-
-#include <libgda/sql-delimiter/gda-sql-delimiter.h>
 
 #define PARENT_TYPE GDA_TYPE_SERVER_PROVIDER
 
@@ -49,7 +44,7 @@ static GObjectClass *parent_class = NULL;
 static void gda_freetds_provider_class_init (GdaFreeTDSProviderClass *klass);
 static void gda_freetds_provider_init       (GdaFreeTDSProvider      *provider,
                                              GdaFreeTDSProviderClass *klass);
-static void gda_freetds_provider_finalize   (GObject *object);
+static void gda_freetds_provider_finalize   (GObject                 *object);
 
 static gboolean gda_freetds_provider_open_connection (GdaServerProvider *provider,
                                                       GdaConnection *cnc,
@@ -63,6 +58,12 @@ static const gchar *gda_freetds_provider_get_database (GdaServerProvider *provid
 static gboolean gda_freetds_provider_change_database (GdaServerProvider *provider,
                                                       GdaConnection *cnc,
                                                       const gchar *name);
+static gboolean gda_freetds_provider_create_database (GdaServerProvider *provider,
+                                                      GdaConnection *cnc,
+                                                      const gchar *name);
+static gboolean gda_freetds_provider_drop_database (GdaServerProvider *provider,
+                                                    GdaConnection *cnc,
+                                                    const gchar *name);
 static GList *gda_freetds_provider_execute_command (GdaServerProvider *provider,
                                                     GdaConnection *cnc,
                                                     GdaCommand *cmd,
@@ -115,13 +116,13 @@ static gboolean gda_freetds_execute_cmd (GdaConnection *cnc, const gchar *sql);
 static int gda_freetds_provider_tds_handle_message (void *aStruct,
 						    void *bStruct,
                                                     const gboolean is_err_msg);
-#if defined(HAVE_FREETDS_VER0_63) || defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_60)
+#if defined(HAVE_FREETDS_VER0_6X) || defined(HAVE_FREETDS_VER0_60)
   static int gda_freetds_provider_tds_handle_info_msg (TDSCONTEXT *,
                                                        TDSSOCKET *,
-                                                       _TDSMSGINFO *);
+                                                       TDSMSGINFO *);
   static int gda_freetds_provider_tds_handle_err_msg (TDSCONTEXT *,
                                                       TDSSOCKET *,
-                                                      _TDSMSGINFO *);
+                                                      TDSMSGINFO *);
 #else
   static int gda_freetds_provider_tds_handle_info_msg (void *aStruct);
   static int gda_freetds_provider_tds_handle_err_msg (void *aStruct);
@@ -152,7 +153,7 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	const gchar *tds_dump = NULL;
 	const gchar *tds_dumpconfig = NULL;
 
-	GdaConnectionEvent *error = NULL;
+	GdaError *error = NULL;
 	GdaFreeTDSProvider *tds_provider = (GdaFreeTDSProvider *) provider;
 	GdaFreeTDSConnectionData *tds_cnc = NULL;
 
@@ -194,7 +195,7 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	/* FreeTDS SIGSEGV on NULL pointers */
 	if ((t_user == NULL) || (t_host == NULL) || (t_password == NULL)) {
 		error = gda_freetds_make_error(NULL, _("Connection aborted. You must provide at least a host, username and password using DSN 'QUERY=;USER=;PASSWORD='."));
-		gda_connection_add_event(cnc, error);
+		gda_connection_add_error(cnc, error);
 
 		return FALSE;
 	}
@@ -206,14 +207,14 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	if (tds_cnc->msg_arr == NULL) {
 		gda_freetds_free_connection_data (tds_cnc);
 		tds_cnc = NULL;
-		gda_connection_add_event_string (cnc, _("Error creating message container."));
+		gda_connection_add_error_string (cnc, _("Error creating message container."));
 		return FALSE;
 	}
 
 	tds_cnc->err_arr = g_ptr_array_new ();
 	if (tds_cnc->err_arr == NULL) {
 		gda_freetds_free_connection_data (tds_cnc);
-		gda_connection_add_event_string (cnc, _("Error creating error container."));
+		gda_connection_add_error_string (cnc, _("Error creating error container."));
 		return FALSE;
 	}
 
@@ -251,13 +252,13 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	tds_set_packet(tds_cnc->login, 512);
 
 	/* Version 0.60 api uses context additionaly */
-#if defined(HAVE_FREETDS_VER0_63) || defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_60)
+#if defined(HAVE_FREETDS_VER0_6X) || defined(HAVE_FREETDS_VER0_60)
 	tds_cnc->ctx = tds_alloc_context();
 	if (! tds_cnc->ctx) {
 		gda_log_error (_("Allocating tds context failed."));
 		gda_freetds_free_connection_data (tds_cnc);
 		error = gda_freetds_make_error(NULL, _("Allocating tds context failed."));
-		gda_connection_add_event(cnc, error);
+		gda_connection_add_error(cnc, error);
 		return FALSE;
 	}
 	/* Initialize callbacks which are now in context struct */
@@ -268,12 +269,12 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	/* establish connection; change in 0.6x api */
 #if defined(HAVE_FREETDS_VER0_60)
 	tds_cnc->tds = tds_connect(tds_cnc->login, tds_cnc->ctx, NULL);
-#elif defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
+#elif defined(HAVE_FREETDS_VER0_6X)
 	tds_cnc->tds = tds_alloc_socket(tds_cnc->ctx, 512);
 	if (! tds_cnc->tds) {
 		gda_log_error (_("Allocating tds socket failed."));
 		gda_freetds_free_connection_data (tds_cnc);
-		gda_connection_add_event_string (cnc, _("Allocating tds socket failed."));
+		gda_connection_add_error_string (cnc, _("Allocating tds socket failed."));
 		return FALSE;
 	}
 	tds_set_parent (tds_cnc->tds, NULL);
@@ -281,7 +282,7 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	if (tds_connect (tds_cnc->tds, tds_cnc->config) != TDS_SUCCEED) {
 		gda_log_error (_("Establishing connection failed."));
 		//gda_freetds_free_connection_data (tds_cnc);
-		gda_connection_add_event_string (cnc, _("Establishing connection failed."));
+		gda_connection_add_error_string (cnc, _("Establishing connection failed."));
 		return FALSE;
 	}
 #else
@@ -291,12 +292,12 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 		gda_log_error (_("Establishing connection failed."));
 		gda_freetds_free_connection_data (tds_cnc);
 		error = gda_freetds_make_error(NULL, _("Establishing connection failed."));
-		gda_connection_add_event(cnc, error);
+		gda_connection_add_error(cnc, error);
 		return FALSE;
 	}
 
 	/* try to receive connection info for sanity check */
-#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
+#if defined(HAVE_FREETDS_VER0_6X)
 	/* do nothing */
 #elif defined(HAVE_FREETDS_VER0_60)
 	tds_cnc->config = tds_get_config(tds_cnc->tds, tds_cnc->login, tds_cnc->ctx->locale);
@@ -306,7 +307,7 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	if (! tds_cnc->config) {
 		gda_log_error (_("Failed getting connection info."));	
 		error = gda_freetds_make_error (tds_cnc->tds, _("Failed getting connection info."));
-		gda_connection_add_event (cnc, error);
+		gda_connection_add_error (cnc, error);
 
 		gda_freetds_free_connection_data (tds_cnc);
 		return FALSE;
@@ -356,7 +357,7 @@ gda_freetds_free_connection_data (GdaFreeTDSConnectionData *tds_cnc)
 		tds_cnc->database = NULL;
 	}
 	if (tds_cnc->config) {
-#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
+#ifdef HAVE_FREETDS_VER0_6X
 		tds_free_connect (tds_cnc->config);
 #else
 		tds_free_config(tds_cnc->config);
@@ -369,7 +370,7 @@ gda_freetds_free_connection_data (GdaFreeTDSConnectionData *tds_cnc)
 		tds_free_socket (tds_cnc->tds);
 		tds_cnc->tds = NULL;
 	}
-#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63) || defined(HAVE_FREETDS_VER0_60)
+#if defined(HAVE_FREETDS_VER0_6X) || defined(HAVE_FREETDS_VER0_60)
 	if (tds_cnc->ctx) {
 		/* Clear callback handler */
 		tds_cnc->ctx->msg_handler = NULL;
@@ -480,6 +481,32 @@ gda_freetds_provider_change_database (GdaServerProvider *provider,
 	return ret;
 }
 
+static gboolean
+gda_freetds_provider_create_database (GdaServerProvider *provider,
+                                      GdaConnection *cnc,
+                                      const gchar *name)
+{
+	GdaFreeTDSProvider *tds_prov = (GdaFreeTDSProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+
+	return FALSE;
+}
+
+static gboolean
+gda_freetds_provider_drop_database (GdaServerProvider *provider,
+                                    GdaConnection *cnc,
+                                    const gchar *name)
+{
+	GdaFreeTDSProvider *tds_prov = (GdaFreeTDSProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	
+	return FALSE;
+}
+
 static GList
 *gda_freetds_provider_execute_command (GdaServerProvider *provider,
                                        GdaConnection *cnc,
@@ -501,11 +528,12 @@ static GList
 		case GDA_COMMAND_TYPE_TABLE:
 			query = g_strdup_printf ("SELECT * FROM %s", gda_command_get_text (cmd));
 			reclist = gda_freetds_provider_process_sql_commands (reclist, cnc, query);
-			if (reclist && GDA_IS_DATA_MODEL (reclist->data)) 
-				g_object_set (G_OBJECT (reclist->data), 
-					      "command_text", gda_command_get_text (cmd),
-					      "command_type", GDA_COMMAND_TYPE_TABLE, NULL);
-
+			if (reclist && GDA_IS_DATA_MODEL (reclist->data)) {
+				gda_data_model_set_command_text (GDA_DATA_MODEL (reclist->data),
+				                                 gda_command_get_text (cmd));
+				gda_data_model_set_command_type (GDA_DATA_MODEL (reclist->data),
+				                                 GDA_COMMAND_TYPE_TABLE);
+			}
 			g_free(query);
 			query = NULL;
 			break;
@@ -598,11 +626,7 @@ static const gchar
 	GdaDataModel *model = NULL;
 	
 	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), NULL);
-	if (cnc)
-		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-	else
-		return NULL;
-
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	tds_cnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_FREETDS_HANDLE);
 	g_return_val_if_fail (tds_cnc != NULL, NULL);
 
@@ -611,11 +635,11 @@ static const gchar
 		if (model) {
 			if ((gda_data_model_get_n_columns (model) == 1)
 			    && (gda_data_model_get_n_rows (model) == 1)) {
-				GValue *value;
+				GdaValue *value;
 
-				value = (GValue *) gda_data_model_get_value_at
+				value = (GdaValue *) gda_data_model_get_value_at
 				                             (model, 0, 0);
-				tds_cnc->server_id = gda_value_stringify ((GValue *) value);
+				tds_cnc->server_id = gda_value_stringify ((GdaValue *) value);
 			}
 			g_object_unref (model);
 		}
@@ -634,14 +658,14 @@ static const gchar
 	return PACKAGE_VERSION;
 }
 
-static GdaDataModel *
-gda_freetds_provider_get_types (GdaConnection    *cnc,
-				GdaParameterList *params)
+static GdaDataModel
+*gda_freetds_provider_get_types (GdaConnection    *cnc,
+                                 GdaParameterList *params)
 {
 	GdaDataModel *model = NULL;
-	_TDSCOLINFO col;
-	GType g_type;
-	GValue     *value = NULL;
+	TDSCOLINFO col;
+	GdaValueType gda_type;
+	GdaValue     *value = NULL;
 	gint i = 1;
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -652,30 +676,38 @@ gda_freetds_provider_get_types (GdaConnection    *cnc,
 	
 	if (model) {
 		for (i = 0; i < gda_data_model_get_n_rows (model); i++) {
-			GdaRow *row = (GdaRow *) gda_data_model_get_row (model, i);
+			GdaRow *row = (GdaRow *) gda_data_model_get_row (model,
+			                                                 i);
 			
-			/* first fix g_type */
+			/* first fix gda_type */
 			if (row) {
 				value = gda_row_get_value (row, 2);
-				if (G_VALUE_TYPE (value) == G_TYPE_INT)
-					col.column_size = g_value_get_int (value);
-				else
+				
+				if (gda_value_get_type (value) == GDA_VALUE_TYPE_INTEGER) {
+					col.column_size = gda_value_get_integer (value);
+				} else {
 					col.column_size = 0;
-				
+				}
 				value = gda_row_get_value (row, 3);
-				if (G_VALUE_TYPE (value) != G_TYPE_CHAR)
+				if (gda_value_get_type (value) != GDA_VALUE_TYPE_TINYINT) {
 					col.column_type = SYBVARIANT;
-				else 
-					col.column_type = g_value_get_char (value);
+				} else {
+					col.column_type = gda_value_get_tinyint (value);
+				}
 				
-				g_type = gda_freetds_get_value_type (&col);
+				gda_type = gda_freetds_get_value_type (&col);
+				/* col 3: type -> clear and set
+				 *   step 1: make sure value is cleared
+				 */
+				gda_value_set_null (value);
+				memset (value, 0, sizeof(GdaValue));
+				/*   step 2: set type of gdavalue */
+				value->type = GDA_VALUE_TYPE_TYPE;
+				value->value.v_type = gda_type;
 				
-				/* col 3: type */
-				g_value_set_ulong (value, g_type);
-				
-				/* col 2: comment */
+				/* col 2: comment -> clear */
 				value = gda_row_get_value (row, 2);
-				g_value_set_string (value, "");
+				gda_value_set_string (value, "");
 			}
 		}
 	}
@@ -683,9 +715,8 @@ gda_freetds_provider_get_types (GdaConnection    *cnc,
 	return model;
 }
 
-static GdaDataModel *
-gda_freetds_provider_get_schema (GdaServerProvider *provider,
-				  GdaConnection *cnc,
+static GdaDataModel
+*gda_freetds_provider_get_schema (GdaServerProvider *provider,                                                    GdaConnection *cnc,
                                   GdaConnectionSchema schema,
                                   GdaParameterList *params)
 {
@@ -694,10 +725,7 @@ gda_freetds_provider_get_schema (GdaServerProvider *provider,
 	GdaDataModel *recset = NULL;
 	
 	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), NULL);
-	if (cnc)
-		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-	else
-		return NULL;
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
 	switch (schema) {
 		case GDA_CONNECTION_SCHEMA_DATABASES:
@@ -753,8 +781,8 @@ static gboolean
 gda_freetds_execute_cmd (GdaConnection *cnc, const gchar *sql)
 {
 	GdaFreeTDSConnectionData *tds_cnc;
-	GdaConnectionEvent *error;
-#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
+	GdaError *error;
+#ifdef HAVE_FREETDS_VER0_6X
 	int result_type = 0;
 #endif
 
@@ -769,13 +797,13 @@ gda_freetds_execute_cmd (GdaConnection *cnc, const gchar *sql)
 	if (tds_cnc->rc != TDS_SUCCEED) {
 		gda_log_error (_("Query did not succeed in execute_cmd()."));
 		error = gda_freetds_make_error (tds_cnc->tds, _("Query did not succeed in execute_cmd()."));
-		gda_connection_add_event (cnc, error);
+		gda_connection_add_error (cnc, error);
 		return FALSE;
 	}
 
 	/* there should not be any result tokens */
-#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
-	while ((tds_cnc->rc = tds_process_result_tokens (tds_cnc->tds, &tds_cnc->result_type, NULL))
+#ifdef HAVE_FREETDS_VER0_6X
+	while ((tds_cnc->rc = tds_process_result_tokens (tds_cnc->tds, &tds_cnc->result_type))
 #else
 	while ((tds_cnc->rc = tds_process_result_tokens (tds_cnc->tds)) 
 #endif
@@ -784,7 +812,7 @@ gda_freetds_execute_cmd (GdaConnection *cnc, const gchar *sql)
 			gda_log_error (_("Unexpected result tokens in execute_cmd()."));
 			error = gda_freetds_make_error (tds_cnc->tds,
 			                                _("Unexpected result tokens in execute_cmd()."));
-			gda_connection_add_event (cnc, error);
+			gda_connection_add_error (cnc, error);
 			return FALSE;
 		}
 	}
@@ -793,7 +821,7 @@ gda_freetds_execute_cmd (GdaConnection *cnc, const gchar *sql)
 		error = gda_freetds_make_error (tds_cnc->tds,
 		                                _("Unexpected return in execute_cmd()."));
 		gda_log_error (_("Unexpected return in execute_cmd()."));
-		gda_connection_add_event (cnc, error);
+		gda_connection_add_error (cnc, error);
 		return FALSE;
 	}
 
@@ -804,7 +832,7 @@ static GdaDataModel *
 gda_freetds_execute_query (GdaConnection *cnc, const gchar* sql)
 {
 	GdaFreeTDSConnectionData *tds_cnc;
-	GdaConnectionEvent *error;
+	GdaError *error;
 	GdaDataModel *recset;
 	
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
@@ -816,14 +844,14 @@ gda_freetds_execute_query (GdaConnection *cnc, const gchar* sql)
 
 	if (tds_cnc->rc != TDS_SUCCEED) {
 		error = gda_freetds_make_error (tds_cnc->tds, NULL);
-		gda_connection_add_event (cnc, error);
+		gda_connection_add_error (cnc, error);
 		return NULL;
 	}
 	recset = gda_freetds_recordset_new (cnc, TRUE);
-	if (GDA_IS_FREETDS_RECORDSET (recset)) 
-		g_object_set (G_OBJECT (recset), 
-			      "command_text", sql,
-			      "command_type", GDA_COMMAND_TYPE_SQL, NULL);
+	if (GDA_IS_FREETDS_RECORDSET (recset)) {
+		gda_data_model_set_command_text (recset, sql);
+		gda_data_model_set_command_type (recset, GDA_COMMAND_TYPE_SQL);
+	}
 
 	return recset;
 }
@@ -836,19 +864,19 @@ gda_freetds_get_stringresult_of_query (GdaConnection *cnc,
                                        const gint row)
 {
 	GdaDataModel    *model = NULL;
-	GValue        *value = NULL;
+	GdaValue        *value = NULL;
 	gchar           *ret = NULL;
 
 	/* GDA_IS_CONNECTION (cnc) validation in function call */
 	model = gda_freetds_execute_query (cnc, sql);
 	
 	if (model) {
-		value = (GValue *) gda_data_model_get_value_at (model,
+		value = (GdaValue *) gda_data_model_get_value_at (model,
 		                                                  col, row);
 
 		if ((value != NULL)
-		    && (G_VALUE_TYPE (value) != GDA_TYPE_NULL)
-		    && (G_VALUE_TYPE (value) != G_TYPE_INVALID)) {
+		    && (gda_value_get_type (value) != GDA_VALUE_TYPE_NULL)
+		    && (gda_value_get_type (value) != GDA_VALUE_TYPE_UNKNOWN)) {
 			ret = (gchar *) gda_value_stringify (value);
 		}
 		
@@ -891,7 +919,7 @@ gda_freetds_get_fields (GdaConnection *cnc, GdaParameterList *params)
 	parameter = gda_parameter_list_find (params, "name");
 	g_return_val_if_fail (parameter != NULL, NULL);
 
-	table = g_value_get_string ((GValue *) gda_parameter_get_value (parameter));
+	table = gda_value_get_string ((GdaValue *) gda_parameter_get_value (parameter));
 	g_return_val_if_fail (table != NULL, NULL);
 
 	query = g_strdup_printf (TDS_SCHEMA_FIELDS, table);
@@ -911,7 +939,7 @@ static GList* gda_freetds_provider_process_sql_commands(GList         *reclist,
                                                         const gchar   *sql)
 {
 	GdaFreeTDSConnectionData *tds_cnc;
-	GdaConnectionEvent *error;
+	GdaError *error;
 	gchar **arr;
 	
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
@@ -920,7 +948,7 @@ static GList* gda_freetds_provider_process_sql_commands(GList         *reclist,
 	g_return_val_if_fail (tds_cnc->tds != NULL, NULL);
 
 /*	arr = gda_freetds_split_commandlist(sql); */
-	arr = gda_delimiter_split_sql (sql);
+	arr = g_strsplit (sql, ";", 0);
 	if (arr) {
 		gint n = 0;
 		while (arr[n]) {
@@ -930,14 +958,14 @@ static GList* gda_freetds_provider_process_sql_commands(GList         *reclist,
 			if (tds_cnc->rc != TDS_SUCCEED) {
 				error = gda_freetds_make_error(tds_cnc->tds,
 						               NULL);
-				gda_connection_add_event (cnc, error);
+				gda_connection_add_error (cnc, error);
 			}
 			recset = gda_freetds_recordset_new (cnc, TRUE);
-			if (GDA_IS_FREETDS_RECORDSET (recset))
-				g_object_set (G_OBJECT (recset), 
-					      "command_text", arr[n],
-					      "command_type", GDA_COMMAND_TYPE_SQL, NULL);
+			if (GDA_IS_FREETDS_RECORDSET (recset)) {
+				gda_data_model_set_command_text (recset, arr[n]);
+				gda_data_model_set_command_type (recset, GDA_COMMAND_TYPE_SQL);
 				reclist = g_list_append (reclist, recset);
+                        }
 
                         n++;
 
@@ -959,37 +987,20 @@ gda_freetds_provider_class_init (GdaFreeTDSProviderClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_freetds_provider_finalize;
-
-	provider_class->get_version = gda_freetds_provider_get_version;
-	provider_class->get_server_version = gda_freetds_provider_get_server_version;
-	provider_class->get_info = NULL;
-	provider_class->supports_feature = gda_freetds_provider_supports;
-	provider_class->get_schema = gda_freetds_provider_get_schema;
-
-	provider_class->get_data_handler = NULL;
-	provider_class->string_to_value = NULL;
-	provider_class->get_def_dbms_type = NULL;
-
 	provider_class->open_connection = gda_freetds_provider_open_connection;
 	provider_class->close_connection = gda_freetds_provider_close_connection;
 	provider_class->get_database = gda_freetds_provider_get_database;
 	provider_class->change_database = gda_freetds_provider_change_database;
-
-	provider_class->supports_operation = NULL;
-        provider_class->create_operation = NULL;
-        provider_class->render_operation = NULL;
-        provider_class->perform_operation = NULL;
-
+	provider_class->create_database = gda_freetds_provider_create_database;
+	provider_class->drop_database = gda_freetds_provider_drop_database;
 	provider_class->execute_command = gda_freetds_provider_execute_command;
-	provider_class->get_last_insert_id = NULL;
-
 	provider_class->begin_transaction = gda_freetds_provider_begin_transaction;
 	provider_class->commit_transaction = gda_freetds_provider_commit_transaction;
 	provider_class->rollback_transaction = gda_freetds_provider_rollback_transaction;
-	
-	provider_class->create_blob = NULL;
-	provider_class->fetch_blob = NULL;	
-	
+	provider_class->get_version = gda_freetds_provider_get_version;
+	provider_class->get_server_version = gda_freetds_provider_get_server_version;
+	provider_class->supports = gda_freetds_provider_supports;
+	provider_class->get_schema = gda_freetds_provider_get_schema;
 
 #ifdef HAVE_FREETDS_VER0_5X
 	if (tds_cbs_initialized == FALSE) {
@@ -1032,10 +1043,10 @@ static int gda_freetds_provider_tds_handle_message (void *aStruct,
                                                     const gboolean is_err_msg)
 {
 	TDSSOCKET *tds = (TDSSOCKET *) aStruct;
-	_TDSMSGINFO *msg_info = (_TDSMSGINFO *) bStruct;
+	TDSMSGINFO *msg_info = (TDSMSGINFO *) bStruct;
 	GdaConnection *cnc = NULL;
 	GdaFreeTDSConnectionData *tds_cnc = NULL;
-	GdaConnectionEvent *error = NULL;
+	GdaError *error = NULL;
 	gchar *msg = NULL;
 
 	g_return_val_if_fail (tds != NULL, TDS_SUCCEED);
@@ -1057,18 +1068,18 @@ static int gda_freetds_provider_tds_handle_message (void *aStruct,
 	/* if errormessage, proceed */
 	if (is_err_msg == TRUE) {
 		if (cnc != NULL) {
-			error = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
-			gda_connection_event_set_description (error, msg);
-			gda_connection_event_set_code (error, msg_info->msg_number);
-			gda_connection_event_set_source (error, "gda-freetds");
+			error = gda_error_new ();
+			gda_error_set_description (error, msg);
+			gda_error_set_number (error, msg_info->msg_number);
+			gda_error_set_source (error, "gda-freetds");
 			if (msg_info->sql_state != NULL) {
-				gda_connection_event_set_sqlstate (error,
+				gda_error_set_sqlstate (error,
 				                        msg_info->sql_state);
 			} else {
-				gda_connection_event_set_sqlstate (error, _("Not available"));
+				gda_error_set_sqlstate (error, _("Not available"));
 			}
 
-			gda_connection_add_event (cnc, error);
+			gda_connection_add_error (cnc, error);
 		} else {
 			gda_log_error (msg);
 		}
@@ -1084,14 +1095,14 @@ static int gda_freetds_provider_tds_handle_message (void *aStruct,
 	return TDS_SUCCEED;
 }
 
-#if defined(HAVE_FREETDS_VER0_63) || defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_60)
+#if defined(HAVE_FREETDS_VER0_6X) || defined(HAVE_FREETDS_VER0_60)
 /* FIXME: rewrite tds_handle_message as well/use new parameters here */
 static int
 gda_freetds_provider_tds_handle_info_msg (TDSCONTEXT *ctx, TDSSOCKET *tds,
-                                          _TDSMSGINFO *msg)
+                                          TDSMSGINFO *msg)
 {
 	return gda_freetds_provider_tds_handle_message ((void *) tds,
-#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
+#ifdef HAVE_FREETDS_VER0_6X
 							(void *) msg,
 #else
 							(void *) tds->msg_info,
@@ -1106,14 +1117,14 @@ gda_freetds_provider_tds_handle_info_msg (void *aStruct)
 }
 #endif
 
-#if defined(HAVE_FREETDS_VER0_63) || defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_60)
+#if defined(HAVE_FREETDS_VER0_6X) || defined(HAVE_FREETDS_VER0_60)
 /* FIXME: rewrite tds_handle_message as well/use new parameters here */
 static int
 gda_freetds_provider_tds_handle_err_msg (TDSCONTEXT *ctx, TDSSOCKET *tds,
-                                         _TDSMSGINFO *msg)
+                                         TDSMSGINFO *msg)
 {
 	return gda_freetds_provider_tds_handle_message ((void *) tds,
-#if defined(HAVE_FREETDS_VER0_61_62) || defined(HAVE_FREETDS_VER0_63)
+#ifdef HAVE_FREETDS_VER0_6X
 							(void *) msg,
 #else
 							(void *) tds->msg_info,
