@@ -179,7 +179,7 @@ gda_mdb_provider_new (void)
 static const gchar *
 gda_mdb_provider_get_version (GdaServerProvider *provider)
 {
-	return VERSION;
+	return PACKAGE_VERSION;
 }
 
 /* open_connection handler for the GdaMdbProvider class */
@@ -209,7 +209,7 @@ gda_mdb_provider_open_connection (GdaServerProvider *provider,
 	mdb_cnc = g_new0 (GdaMdbConnection, 1);
 	mdb_cnc->cnc = cnc;
 	mdb_cnc->server_version = NULL;
-	mdb_cnc->mdb = mdb_open (filename);
+	mdb_cnc->mdb = mdb_open (filename, MDB_WRITABLE);
 	if (!mdb_cnc->mdb) {
 		gda_connection_add_error_string (cnc, _("Could not open file %s"), filename);
 		g_free (mdb_cnc);
@@ -389,7 +389,7 @@ gda_mdb_provider_execute_command (GdaServerProvider *provider,
 			g_strfreev (arr);
 		}
 		break;
-	default :
+	default : ;
 	}
 
 	return reclist;
@@ -438,7 +438,7 @@ gda_mdb_provider_supports (GdaServerProvider *provider,
 	case GDA_CONNECTION_FEATURE_PROCEDURES :
 	case GDA_CONNECTION_FEATURE_SQL :
 		return TRUE;
-	default :
+	default : ;
 	}
 
 	return FALSE;
