@@ -1,11 +1,10 @@
 /* GNOME DB Postgres Provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation
+ * Copyright (C) 1998-2002 The GNOME Foundation
  *
  * AUTHORS:
  *         Vivien Malerba <malerba@gnome-db.org>
  *         Rodrigo Moya <rodrigo@gnome-db.org>
  *         Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
- *         Bas Driessen <bas.driessen@xobas.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,7 +24,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <libgda/gda-data-model-array.h>
-#include <libgda/gda-column-index.h>
 #include "gda-postgres.h"
 #include "gda-postgres-provider.h"
 
@@ -43,52 +41,21 @@ static gboolean gda_postgres_provider_open_connection (GdaServerProvider *provid
 
 static gboolean gda_postgres_provider_close_connection (GdaServerProvider *provider,
 							GdaConnection *cnc);
-
 static const gchar *gda_postgres_provider_get_server_version (GdaServerProvider *provider,
 							      GdaConnection *cnc);
-
 static const gchar *gda_postgres_provider_get_database (GdaServerProvider *provider,
 							GdaConnection *cnc);
-
-static gchar *gda_postgres_provider_get_specs_create_database  (GdaServerProvider *provider);
-
-static gboolean gda_postgres_provider_create_database_params (GdaServerProvider *provider, 
-							      GdaParameterList *params, GError **error);
-
-static gboolean gda_postgres_provider_create_database_cnc (GdaServerProvider *provider,
-							   GdaConnection *cnc,
-							   const gchar *name);
-
+static gboolean gda_postgres_provider_create_database (GdaServerProvider *provider,
+						       GdaConnection *cnc,
+						       const gchar *name);
 static gboolean gda_postgres_provider_drop_database (GdaServerProvider *provider,
 						     GdaConnection *cnc,
 						     const gchar *name);
 
-static gboolean gda_postgres_provider_create_table (GdaServerProvider *provider,
-						    GdaConnection *cnc,
-						    const gchar *table_name,
-						    const GList *attributes_list,
-						    const GList *index_list);
-
-static gboolean gda_postgres_provider_drop_table (GdaServerProvider *provider,
-						     GdaConnection *cnc,
-						     const gchar *table_name);
-
-static gboolean gda_postgres_provider_create_index (GdaServerProvider *provider,
-						    GdaConnection *cnc,
-						    const GdaDataModelIndex *index,
-						    const gchar *table_name);
-
-static gboolean gda_postgres_provider_drop_index (GdaServerProvider *provider,
-						     GdaConnection *cnc,
-						     const gchar *index_name,
-						     gboolean primary_key,
-						     const gchar *table_name);
-
 static GList *gda_postgres_provider_execute_command (GdaServerProvider *provider,
 						     GdaConnection *cnc,
 						     GdaCommand *cmd,
 						     GdaParameterList *params);
-
 static gchar *gda_postgres_provider_get_last_insert_id (GdaServerProvider *provider,
 							GdaConnection *cnc,
 							GdaDataModel *recset);
@@ -121,11 +88,10 @@ static GdaDataModel *gda_postgres_provider_get_schema (GdaServerProvider *provid
 static gboolean gda_postgres_provider_create_blob (GdaServerProvider *provider,
 						   GdaConnection *cnc,
 						   GdaBlob *blob);
-
-gchar *
-gda_postgres_provider_value_to_sql_string (GdaServerProvider *provider, /* we dont actually use this!*/
-					GdaConnection *cnc,
-					GdaValue *from);
+static gboolean gda_postgres_provider_escape_string (GdaServerProvider *provider,
+						     GdaConnection *cnc,
+						     const gchar *from,
+						     gchar *to);
 				 		 
 typedef struct {
 	gchar *col_name;
@@ -169,14 +135,8 @@ gda_postgres_provider_class_init (GdaPostgresProviderClass *klass)
 	provider_class->close_connection = gda_postgres_provider_close_connection;
 	provider_class->get_server_version = gda_postgres_provider_get_server_version;
 	provider_class->get_database = gda_postgres_provider_get_database;
-	provider_class->get_specs_create_database = gda_postgres_provider_get_specs_create_database;
-	provider_class->create_database_params = gda_postgres_provider_create_database_params;
-	provider_class->create_database_cnc = gda_postgres_provider_create_database_cnc;
+	provider_class->create_database = gda_postgres_provider_create_database;
 	provider_class->drop_database = gda_postgres_provider_drop_database;
-	provider_class->create_table = gda_postgres_provider_create_table;
-	provider_class->drop_table = gda_postgres_provider_drop_table;
-	provider_class->create_index = gda_postgres_provider_create_index;
-	provider_class->drop_index = gda_postgres_provider_drop_index;
 	provider_class->execute_command = gda_postgres_provider_execute_command;
 	provider_class->get_last_insert_id = gda_postgres_provider_get_last_insert_id;
 	provider_class->begin_transaction = gda_postgres_provider_begin_transaction;
@@ -185,7 +145,7 @@ gda_postgres_provider_class_init (GdaPostgresProviderClass *klass)
 	provider_class->supports = gda_postgres_provider_supports;
 	provider_class->get_schema = gda_postgres_provider_get_schema;
 	provider_class->create_blob = gda_postgres_provider_create_blob;
-	provider_class->value_to_sql_string = gda_postgres_provider_value_to_sql_string;
+	provider_class->escape_string = gda_postgres_provider_escape_string;
 }
 
 static void
@@ -270,67 +230,6 @@ postgres_name_to_gda_type (const gchar *name)
 	return GDA_VALUE_TYPE_STRING;
 }
 
-static gchar *
-postgres_name_from_gda_type (const GdaValueType type)
-{
-	switch (type) {
-	case GDA_VALUE_TYPE_NULL :
-		return g_strdup_printf ("text");
-	case GDA_VALUE_TYPE_BIGINT :
-		return g_strdup_printf ("int8");
-	case GDA_VALUE_TYPE_BIGUINT :
-		return g_strdup_printf ("int8");
-	case GDA_VALUE_TYPE_BINARY :
-		return g_strdup_printf ("bytea");
-	case GDA_VALUE_TYPE_BLOB :
-		return g_strdup_printf ("oid");
-	case GDA_VALUE_TYPE_BOOLEAN :
-		return g_strdup_printf ("bool");
-	case GDA_VALUE_TYPE_DATE :
-		return g_strdup_printf ("date");
-	case GDA_VALUE_TYPE_DOUBLE :
-		return g_strdup_printf ("float8");
-	case GDA_VALUE_TYPE_GEOMETRIC_POINT :
-		return g_strdup_printf ("point");
-	case GDA_VALUE_TYPE_GOBJECT :
-		return g_strdup_printf ("text");
-	case GDA_VALUE_TYPE_INTEGER :
-		return g_strdup_printf ("int4");
-	case GDA_VALUE_TYPE_LIST :
-		return g_strdup_printf ("text");
-	case GDA_VALUE_TYPE_MONEY :
-		return g_strdup_printf ("money");
-	case GDA_VALUE_TYPE_NUMERIC :
-		return g_strdup_printf ("numeric");
-	case GDA_VALUE_TYPE_SINGLE :
-		return g_strdup_printf ("float4");
-	case GDA_VALUE_TYPE_SMALLINT :
-		return g_strdup_printf ("int2");
-	case GDA_VALUE_TYPE_SMALLUINT :
-		return g_strdup_printf ("int2");
-	case GDA_VALUE_TYPE_STRING :
-		return g_strdup_printf ("varchar");
-	case GDA_VALUE_TYPE_TIME :
-		return g_strdup_printf ("time");
-	case GDA_VALUE_TYPE_TIMESTAMP :
-		return g_strdup_printf ("timestamp");
-	case GDA_VALUE_TYPE_TINYINT :
-		return g_strdup_printf ("smallint");
-	case GDA_VALUE_TYPE_TINYUINT :
-		return g_strdup_printf ("smallint");
-	case GDA_VALUE_TYPE_TYPE :
-		return g_strdup_printf ("int2");
-        case GDA_VALUE_TYPE_UINTEGER :
-		return g_strdup_printf ("int4");
-	case GDA_VALUE_TYPE_UNKNOWN :
-		return g_strdup_printf ("text");
-	default :
-		return g_strdup_printf ("text");
-	}
-
-	return g_strdup_printf ("text");
-}
-
 static int
 get_connection_type_list (GdaPostgresConnectionData *priv_td)
 {
@@ -781,161 +680,11 @@ gda_postgres_provider_get_database (GdaServerProvider *provider,
 	return (const char *) PQdb ((const PGconn *) priv_data->pconn);
 }
 
-/* get_specs_create_database handler for the GdaPostgresProvider class */ 
-static gchar *
-gda_postgres_provider_get_specs_create_database (GdaServerProvider *provider)
-{
-	gchar *specs, *file;
-	gint len;
-	GdaPostgresProvider *pg_prv = (GdaPostgresProvider *) provider;
-
-	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (pg_prv), NULL);
-
-	file = g_build_filename (LIBGDA_DATA_DIR, "postgres_specs_create_db.xml", NULL);
-	if (g_file_get_contents (file, &specs, &len, NULL)) 
-		return specs;
-	else
-		return NULL;
-}
-
-#define string_from_string_param(param) \
-	(param && gda_parameter_get_value (param) && !gda_value_is_null (gda_parameter_get_value (param))) ? \
-	gda_value_get_string (gda_parameter_get_value (param)) : NULL
-
-#define int_from_int_param(param) \
-	(param && gda_parameter_get_value (param) && !gda_value_is_null (gda_parameter_get_value (param))) ? \
-	gda_value_get_integer (gda_parameter_get_value (param)) : -1
-
-#define bool_from_bool_param(param) \
-	(param && gda_parameter_get_value (param) && !gda_value_is_null (gda_parameter_get_value (param))) ? \
-	gda_value_get_boolean (gda_parameter_get_value (param)) : FALSE
-
-
-/* create_database_params handler for the GdaPostgresProvider class */ 
+/* create_database handler for the GdaPostgresProvider class */
 static gboolean
-gda_postgres_provider_create_database_params (GdaServerProvider *provider,
-					      GdaParameterList *params, GError **error)
-{
-	const gchar *pq_host;
-	const gchar *pq_db;
-	gint         pq_port = -1;
-	const gchar *pq_options = NULL;
-	const gchar *pq_user = NULL;
-	const gchar *pq_pwd = NULL;
-	gboolean     pq_ssl = FALSE;
-
-	const gchar *pq_newdb = NULL;
-	const gchar *pq_enc = NULL;
-	const gchar *pq_owner = NULL;
-	const gchar *pq_tspace = NULL;
-
-	GString *string;
-	PGconn *pconn;
-	PGresult *pg_res;
-	
-	gboolean retval = TRUE;
-	GdaParameter *param = NULL;
-	GdaPostgresProvider *pg_prv = (GdaPostgresProvider *) provider;
-	
-	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (pg_prv), FALSE);
-
-	if (params) {
-		param = gda_parameter_list_find (params, "HOST");
-		pq_host = string_from_string_param (param);
-
-		param = gda_parameter_list_find (params, "PORT");
-		pq_port = int_from_int_param (param);
-
-		param = gda_parameter_list_find (params, "OPTIONS");
-		pq_options = string_from_string_param (param);
-		
-		param = gda_parameter_list_find (params, "ADM_LOGIN");
-		pq_user = string_from_string_param (param);
-
-		param = gda_parameter_list_find (params, "ADM_PASSWORD");
-		pq_pwd = string_from_string_param (param);
-
-		param = gda_parameter_list_find (params, "TEMPLATE");
-		pq_db = string_from_string_param (param);
-
-		param = gda_parameter_list_find (params, "REQUIRESSL");
-		pq_ssl = bool_from_bool_param (param);
-
-		param = gda_parameter_list_find (params, "DATABASE");
-		pq_newdb = string_from_string_param (param);
-
-		param = gda_parameter_list_find (params, "ENCODING");
-		pq_enc = string_from_string_param (param);
-
-		param = gda_parameter_list_find (params, "OWNER");
-		pq_owner = string_from_string_param (param);
-
-		param = gda_parameter_list_find (params, "TABLESPACE");
-		pq_tspace = string_from_string_param (param);
-	}
-	if (!pq_newdb) {
-		g_set_error (error, 0, 0,
-			     _("Missing parameter 'DATABASE'"));
-		return FALSE;
-	}
-
-	/* open a connection with the administrator settings */
-	string = g_string_new ("");
-	if (pq_host && *pq_host)
-		g_string_append_printf (string, "host=%s", pq_host);
-	if (pq_port > 0)
-		g_string_append_printf (string, " port=%d", pq_port);
-	g_string_append_printf (string, " dbname=%s", pq_db ? pq_db : "template1");
-	if (pq_options && *pq_options)
-		g_string_append_printf (string, " options=%s", pq_options);
-	if (pq_user && *pq_user)
-		g_string_append_printf (string, " user=%s", pq_user);
-	if (pq_pwd && *pq_pwd)
-		g_string_append_printf (string, " password=%s", pq_pwd);
-	if (pq_ssl)
-		g_string_append (string, " requiressl=1");
-
-	pconn = PQconnectdb (string->str);
-	g_string_free (string, TRUE);
-	
-	if (PQstatus (pconn) != CONNECTION_OK) {
-		g_set_error (error, 0, 0, PQerrorMessage (pconn));
-		PQfinish(pconn);
-
-		return FALSE;
-	}
-
-	/* Ask to create a database */
-	string = g_string_new ("CREATE DATABASE ");
-	g_string_append (string, pq_newdb);
-	if (pq_owner)
-		g_string_append_printf (string, " OWNER=%s", pq_owner);
-	if (pq_db)
-		g_string_append_printf (string, " TEMPLATE=%s", pq_db);
-	if (pq_enc)
-		g_string_append_printf (string, " ENCODING='%s'", pq_enc);
-	if (pq_tspace)
-		g_string_append_printf (string, " TABLESPACE=%s", pq_tspace);
-
-	pg_res = PQexec (pconn, string->str);
-	g_string_free (string, TRUE);
-
-	if (!pg_res || PQresultStatus (pg_res) != PGRES_COMMAND_OK) {
-		g_set_error (error, 0, 0, PQresultErrorMessage (pg_res));
-		retval = FALSE;
-	}
-
-	PQfinish(pconn);
-	
-	return retval;
-}
-
-
-/* create_database_cnc handler for the GdaPostgresProvider class */
-static gboolean
-gda_postgres_provider_create_database_cnc (GdaServerProvider *provider,
-					   GdaConnection *cnc,
-					   const gchar *name)
+gda_postgres_provider_create_database (GdaServerProvider *provider,
+				       GdaConnection *cnc,
+				       const gchar *name)
 {
 	gboolean retval;
 	gchar *sql;
@@ -973,260 +722,6 @@ gda_postgres_provider_drop_database (GdaServerProvider *provider,
 	return retval;
 }
 
-/* create_table handler for the GdaPostgresProvider class */
-static gboolean
-gda_postgres_provider_create_table (GdaServerProvider *provider,
-				    GdaConnection *cnc,
-				    const gchar *table_name,
-				    const GList *attributes_list,
-				    const GList *index_list)
-{
-	GdaPostgresProvider *pg_prv = (GdaPostgresProvider *) provider;
-	GdaColumn *dmca;
-	GdaDataModelIndex *dmi;
-	GString *sql;
-	gint i;
-	gchar *postgres_data_type, *default_value, *references;
-	GdaValueType value_type;
-	gboolean retval;
-
-	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (pg_prv), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (table_name != NULL, FALSE);
-	g_return_val_if_fail (attributes_list != NULL, FALSE);
-
-	/* prepare the SQL command */
-	sql = g_string_new ("CREATE TABLE ");
-	g_string_append_printf (sql, "%s (", table_name);
-
-	/* step through list */
-	for (i=0; i<g_list_length ((GList *) attributes_list); i++) {
-
-		if (i>0)
-			g_string_append_printf (sql, ", ");
-			
-		dmca = (GdaColumn *) g_list_nth_data ((GList *) attributes_list, i);
-
-		/* name */	
-		g_string_append_printf (sql, "\"%s\" ", gda_column_get_name (dmca));
-
-		/* data type; force serial to be used when auto_increment is set */
-		if (gda_column_get_auto_increment (dmca) == TRUE) {
-			g_string_append_printf (sql, "serial");
-			value_type = GDA_VALUE_TYPE_INTEGER;
-		} else {
-			value_type = gda_column_get_gdatype (dmca);
-			postgres_data_type = postgres_name_from_gda_type (value_type);
-			g_string_append_printf (sql, "%s", postgres_data_type);
-			g_free(postgres_data_type);
-		}
-
-		/* size */
-		if (value_type == GDA_VALUE_TYPE_STRING)
-			g_string_append_printf (sql, "(%ld)", gda_column_get_defined_size (dmca));
-		else if (value_type == GDA_VALUE_TYPE_NUMERIC)
-			g_string_append_printf (sql, "(%ld,%ld)", 
-				gda_column_get_defined_size (dmca),
-				gda_column_get_scale (dmca));
-		
-		/* NULL */
-		if (gda_column_get_allow_null (dmca) == TRUE)
-			g_string_append_printf (sql, " NULL");
-		else
-			g_string_append_printf (sql, " NOT NULL");
-	
-		/* (primary) key */
-		if (gda_column_get_primary_key (dmca) == TRUE)
-			g_string_append_printf (sql, " PRIMARY KEY");
-		else
-			if (gda_column_get_unique_key (dmca) == TRUE)
-				g_string_append_printf (sql, " UNIQUE");
-
-		/* default value (in case of string, user needs to add "'" around the field) */
-		if (gda_column_get_default_value (dmca) != NULL) {
-			default_value = gda_value_stringify ((GdaValue *) gda_column_get_default_value (dmca));
-			if ((default_value != NULL) && (*default_value != '\0'))
-				g_string_append_printf (sql, " DEFAULT %s", default_value);
-		}
-
-		/* any additional parameters */			
-		if (gda_column_get_references (dmca) != NULL) {
-			references = (gchar *) gda_column_get_references (dmca);
-			if ((references != NULL) && (*references != '\0'))
-				g_string_append_printf (sql, " %s", references);
-		}
-	}
-
-	/* finish the SQL command */
-	g_string_append_printf (sql, ")");
-
-	/* execute sql command */
-	retval = gda_postgres_provider_single_command (pg_prv, cnc, sql->str);
-
-	/* clean up */
-	g_string_free (sql, TRUE);
-
-	/* Create indexes */
-	if (index_list != NULL) {
-
-		/* step through index_list */
-		for (i=0; i<g_list_length ((GList *) index_list); i++) {
-
-			/* get index */
-			dmi = (GdaDataModelIndex *) g_list_nth_data ((GList *) index_list, i);
-
-			/* create index */
-			retval = gda_postgres_provider_create_index (provider, cnc, dmi, table_name);
-			if (retval == FALSE)
-				return FALSE;
-		}
-	}
-
-	/* return */
-	return retval;
-}
-
-/* drop_table handler for the GdaPostgresProvider class */
-static gboolean
-gda_postgres_provider_drop_table (GdaServerProvider *provider,
-				  GdaConnection *cnc,
-				  const gchar *table_name)
-{
-	gboolean retval;
-	gchar *sql;
-
-	GdaPostgresProvider *pg_prv = (GdaPostgresProvider *) provider;
-
-	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (pg_prv), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (table_name != NULL, FALSE);
-
-	sql = g_strdup_printf ("DROP TABLE %s", table_name);
-
-	retval = gda_postgres_provider_single_command (pg_prv, cnc, sql);
-	g_free (sql);
-
-	return retval;
-}
-
-/* create_index handler for the GdaPostgresProvider class */
-static gboolean
-gda_postgres_provider_create_index (GdaServerProvider *provider,
-				    GdaConnection *cnc,
-				    const GdaDataModelIndex *index,
-				    const gchar *table_name)
-{
-	GdaPostgresProvider *pg_prv = (GdaPostgresProvider *) provider;
-	GdaColumnIndex *dmcia;
-	GString *sql;
-	GList *col_list;
-	gchar *index_name, *col_ref, *idx_ref;
-	gint i;
-	gboolean retval;
-
-	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (pg_prv), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (index != NULL, FALSE);
-	g_return_val_if_fail (table_name != NULL, FALSE);
-
-	/* the 'CREATE INDEX' command in PostgreSQL is limited to
-	secondary non-primary indices only. Therefore use 'ALTER TABLE'
-	for PRIMARY and UNIQUE */
-
-	/* prepare the SQL command */
-	sql = g_string_new (" ");
-
-	/* get index name */
-	index_name = (gchar *) gda_data_model_index_get_name ((GdaDataModelIndex *) index);
-
-	/* determine type of index (PRIMARY, UNIQUE or INDEX) */
-	if (gda_data_model_index_get_primary_key ((GdaDataModelIndex *) index) == TRUE)
-		g_string_append_printf (sql, "ALTER TABLE %s ADD CONSTRAINT %s PRIMARY KEY (", table_name, index_name);
-	else if (gda_data_model_index_get_unique_key ((GdaDataModelIndex *) index) == TRUE)
-		g_string_append_printf (sql, "ALTER TABLE %s ADD CONSTRAINT %s UNIQUE (", table_name, index_name);
-	else
-		g_string_append_printf (sql, "CREATE INDEX %s ON %s (", index_name, table_name);
-
-	/* get the list of index columns */
-	col_list = gda_data_model_index_get_column_index_list ((GdaDataModelIndex *) index);
-
-	/* step through list */
-	for (i = 0; i < g_list_length ((GList *) col_list); i++) {
-
-		if (i > 0)
-			g_string_append_printf (sql, ", ");
-			
-		dmcia = (GdaColumnIndex *) g_list_nth_data ((GList *) col_list, i);
-
-		/* name */	
-		g_string_append_printf (sql, "\"%s\" ", gda_column_index_get_column_name (dmcia));
-
-		/* any additional column parameters */
-		if (gda_column_index_get_references (dmcia) != NULL) {
-			col_ref = (gchar *) gda_column_index_get_references (dmcia);
-			if ((col_ref != NULL) && (*col_ref != '\0'))
-				g_string_append_printf (sql, " %s ", col_ref);
-		}
-	}
-
-	/* finish the SQL column(s) section command */
-	g_string_append_printf (sql, ")");
-
-	/* any additional index parameters */
-	if (gda_data_model_index_get_references ((GdaDataModelIndex *) index) != NULL) {
-		idx_ref = (gchar *) gda_data_model_index_get_references ((GdaDataModelIndex *) index);
-		if ((idx_ref != NULL) && (*idx_ref != '\0'))
-			g_string_append_printf (sql, " %s ", idx_ref);
-	}
-
-	/* execute sql command */
-	retval = gda_postgres_provider_single_command (pg_prv, cnc, sql->str);
-
-	/* clean up */
-	g_string_free (sql, TRUE);
-
-	return retval;
-}
-
-/* drop_index handler for the GdaPostgresProvider class */
-static gboolean
-gda_postgres_provider_drop_index (GdaServerProvider *provider,
-				  GdaConnection *cnc,
-				  const gchar *index_name,
-				  gboolean primary_key,
-				  const gchar *table_name)
-{
-	gboolean retval = FALSE, retval_temp;
-	gchar *sql_alter;
-	gchar *sql_drop;
-
-	GdaPostgresProvider *pg_prv = (GdaPostgresProvider *) provider;
-
-	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (pg_prv), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (index_name != NULL, FALSE);
-	g_return_val_if_fail (table_name != NULL, FALSE);
-
-	/* In PostgreSQL PRIMARY can only and UNIQUE can as well be defined as
-	   constraints. Therefore use 2 methods for removal */
-	sql_alter = g_strdup_printf ("ALTER TABLE %s DROP CONSTRAINT %s", table_name, index_name);
-	sql_drop  = g_strdup_printf ("DROP INDEX %s", index_name);
-
-	retval_temp = gda_postgres_provider_single_command (pg_prv, cnc, sql_alter);
-	if (retval_temp == TRUE)
-		retval = TRUE;
-
-	retval_temp = gda_postgres_provider_single_command (pg_prv, cnc, sql_drop);
-	if (retval_temp == TRUE)
-		retval = TRUE;
-
-	/* clean up */
-	g_free (sql_alter);
-	g_free (sql_drop);
-
-	return retval;
-}
-
 /* execute_command handler for the GdaPostgresProvider class */
 static GList *
 gda_postgres_provider_execute_command (GdaServerProvider *provider,
@@ -1322,6 +817,7 @@ gda_postgres_provider_begin_transaction (GdaServerProvider *provider,
 		switch (gda_transaction_get_isolation_level (xaction)) {
 		case GDA_TRANSACTION_ISOLATION_READ_COMMITTED :
 		        isolation_level = g_strconcat ("SET TRANSACTION ISOLATION LEVEL READ COMMITTED ", write_option, NULL);
+			g_message ("Begin transaction read committed");
 		        break;
 		case GDA_TRANSACTION_ISOLATION_READ_UNCOMMITTED :
 		        gda_connection_add_error_string (cnc, _("Transactions are not supported in read uncommitted isolation level"));
@@ -1331,6 +827,7 @@ gda_postgres_provider_begin_transaction (GdaServerProvider *provider,
 		        return FALSE;
 		case GDA_TRANSACTION_ISOLATION_SERIALIZABLE :
 		        isolation_level = g_strconcat ("SET TRANSACTION ISOLATION LEVEL SERIALIZABLE ", write_option, NULL);
+			g_message ("Begin transaction serializable");
 		        break;
 		default: 
 		        isolation_level = NULL;
@@ -1340,6 +837,7 @@ gda_postgres_provider_begin_transaction (GdaServerProvider *provider,
 	result = gda_postgres_provider_single_command (pg_prv, cnc, "BEGIN"); 
 	if (result&&isolation_level != NULL) {
 	        result=gda_postgres_provider_single_command (pg_prv, cnc, isolation_level) ;
+		g_message (isolation_level);
 	} 
 	g_free(isolation_level);
 
@@ -1584,7 +1082,7 @@ gda_postgres_fill_procs_data (GdaDataModelArray *recset,
 					if (!gstr) 
 						gstr = g_string_new (thevalue);
 					else
-						g_string_append_printf (gstr, ",%s", thevalue);
+						g_string_append_printf (gstr, " %s", thevalue);
 				}
 					
 				ptr = strtok (NULL, " ");
@@ -1728,50 +1226,28 @@ get_postgres_types (GdaConnection *cnc, GdaParameterList *params)
 	GdaDataModelArray *recset;
 	GdaPostgresConnectionData *priv_data;
 	gint i;
-	static GHashTable *synonyms = NULL;
-
-	if (!synonyms) {
-		synonyms = g_hash_table_new (g_str_hash, g_str_equal);
-
-		g_hash_table_insert (synonyms, "int4", "int,integer");
-		g_hash_table_insert (synonyms, "int8", "bigint");
-		g_hash_table_insert (synonyms, "serial8", "bigserial");
-		g_hash_table_insert (synonyms, "varbit", "bit varying");
-		g_hash_table_insert (synonyms, "bool", "boolean");
-		g_hash_table_insert (synonyms, "varchar", "character varying");
-		g_hash_table_insert (synonyms, "char", "character");
-		g_hash_table_insert (synonyms, "float8", "double precision");
-		g_hash_table_insert (synonyms, "numeric", "decimal");
-		g_hash_table_insert (synonyms, "float4", "real");
-		g_hash_table_insert (synonyms, "int2", "smallint");
-		g_hash_table_insert (synonyms, "serial4", "serial");
-	}
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
 	/* create the recordset */
-	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (5));
+	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (4));
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Type"));
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 1, _("Owner"));
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 2, _("Comments"));
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 3, _("GDA type"));
-	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 4, _("Synonyms"));
 
 	/* fill the recordset */
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_POSTGRES_HANDLE);
 
 	for (i = 0; i < priv_data->ntypes; i++) {
 		GList *value_list = NULL;
-		gchar *syn;
 
 		value_list = g_list_append (value_list, gda_value_new_string (priv_data->type_data[i].name));
 		value_list = g_list_append (value_list, gda_value_new_string (priv_data->type_data[i].owner));
 		value_list = g_list_append (value_list, gda_value_new_string (priv_data->type_data[i].comments));
 		value_list = g_list_append (value_list, gda_value_new_type (priv_data->type_data[i].type));
-		syn = g_hash_table_lookup (synonyms, priv_data->type_data[i].name);
-		value_list = g_list_append (value_list, gda_value_new_string (syn));
 
-		gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
+		gda_data_model_append_row (GDA_DATA_MODEL (recset), value_list);
 
 		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
 		g_list_free (value_list);
@@ -1983,7 +1459,7 @@ gda_postgres_init_md_recset (GdaConnection *cnc)
 {
 	GdaDataModelArray *recset;
 	gint i;
-	GdaPostgresColData cols[] = {
+	GdaPostgresColData cols[9] = {
 		{ N_("Field name")	, GDA_VALUE_TYPE_STRING  },
 		{ N_("Data type")	, GDA_VALUE_TYPE_STRING  },
 		{ N_("Size")		, GDA_VALUE_TYPE_INTEGER },
@@ -1992,8 +1468,7 @@ gda_postgres_init_md_recset (GdaConnection *cnc)
 		{ N_("Primary key?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("Unique index?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("References")	, GDA_VALUE_TYPE_STRING  },
-		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  },
-		{ N_("Extra attributes"), GDA_VALUE_TYPE_STRING  }
+		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  }
 		};
 
 	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (sizeof cols / sizeof cols[0]));
@@ -2298,7 +1773,7 @@ gda_postgres_fill_md_data (const gchar *tblname, GdaDataModelArray *recset,
 			tblname, tblname);
 	else 
 		query = g_strdup_printf (
-			"SELECT a.attname, t.typname, a.atttypmod, t.typlen, a.attnotnull, pg_get_expr (d.adbin, c.oid), a.attnum "
+			"SELECT a.attname, t.typname, a.atttypmod, t.typlen, a.attnotnull, d.adsrc, a.attnum "
 			"FROM pg_catalog.pg_class c "
 			"LEFT JOIN pg_catalog.pg_attribute a ON (a.attrelid = c.oid) "
 			"FULL JOIN pg_catalog.pg_attrdef d ON (a.attnum = d.adnum AND d.adrelid=c.oid) "
@@ -2405,13 +1880,7 @@ gda_postgres_fill_md_data (const gchar *tblname, GdaDataModelArray *recset,
 		else
 			value = gda_value_new_null ();
 	        rowlist = g_list_append (rowlist, value);
-
-		/* Extra attributes */
-		if (strstr (PQgetvalue (pg_res, i, 5), "nextval"))
-			value = gda_value_new_string ("AUTO_INCREMENT");
-		else
-			value = gda_value_new_string ("");
-	        rowlist = g_list_append (rowlist, value);			
+		
 
 		list = g_list_append (list, rowlist);
 		rowlist = NULL;
@@ -2439,7 +1908,7 @@ add_g_list_row (gpointer data, gpointer user_data)
 	GList *rowlist = data;
 	GdaDataModelArray *recset = user_data;
 
-	gda_data_model_append_values (GDA_DATA_MODEL (recset), rowlist);
+	gda_data_model_append_row (GDA_DATA_MODEL (recset), rowlist);
 	g_list_foreach (rowlist, (GFunc) gda_value_free, NULL);
 	g_list_free (rowlist);
 }
@@ -2691,57 +2160,16 @@ static gboolean gda_postgres_provider_create_blob (GdaServerProvider *provider,
 	return gda_postgres_blob_create (blob, cnc);
 }
 
-gchar *
-gda_postgres_provider_value_to_sql_string (
-				GdaServerProvider *provider, /* we dont actually use this!*/
-				GdaConnection *cnc,
-				GdaValue *from)
+gboolean
+gda_postgres_provider_escape_string (GdaServerProvider *provider,
+				     GdaConnection *cnc,
+				     const gchar *from,
+				     gchar *to)
 {
-	gchar *val_str;
-	gchar *ret;
-	
-	g_return_val_if_fail (from != NULL, NULL);
+	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (provider), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (from != NULL, FALSE);
+	g_return_val_if_fail (to != NULL, FALSE);
 
-	val_str = gda_value_stringify (from);
-	if (!val_str)
-		return NULL;
-
-	switch (from->type) {
-		case GDA_VALUE_TYPE_BIGINT :
-		case GDA_VALUE_TYPE_DOUBLE :
-		case GDA_VALUE_TYPE_INTEGER :
-		case GDA_VALUE_TYPE_NUMERIC :
-		case GDA_VALUE_TYPE_SINGLE :
-		case GDA_VALUE_TYPE_SMALLINT :
-		case GDA_VALUE_TYPE_TINYINT :
-			ret = g_strdup (val_str);
-			break;
-		case GDA_VALUE_TYPE_TIMESTAMP:
-		case GDA_VALUE_TYPE_DATE :
-		case GDA_VALUE_TYPE_TIME :
-			ret = g_strdup_printf ("\"%s\"", val_str);
-			break;
-		case GDA_VALUE_TYPE_BINARY:
-		{
-			int qlen;
-			char *quoted;
-			quoted = PQescapeBytea(val_str, (size_t) strlen(val_str), &qlen);
-			ret = g_strdup_printf ("\"%s\"", quoted);
-			PQfreemem(quoted);
-			break;
-		}
-		default :
-		{
-			char *quoted;
-			quoted = ret = g_malloc(strlen(val_str) * 2 + 3);
-			*quoted++     	= '\'';
-			quoted += PQescapeString (quoted, val_str, (size_t) strlen (val_str));
-			*quoted++     	= '\'';
-			*quoted++     	= '\0';
-			ret = g_realloc(ret, quoted - ret + 1);
-		}
-	}
-	g_free (val_str);
-
-	return ret;
+	return (unsigned long)  PQescapeString (to, from, (size_t) strlen (from));
 }
