@@ -1,5 +1,5 @@
 /* GDA Postgres Provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation
+ * Copyright (C) 1998-2002 The GNOME Foundation
  *
  * AUTHORS:
  *         Vivien Malerba <malerba@gnome-db.org>
@@ -22,13 +22,11 @@
  */
 
 #include <libgda/gda-intl.h>
-#include <libgda/gda-config.h>
 #include "gda-sqlite-provider.h"
 
-const gchar       *plugin_get_name (void);
-const gchar       *plugin_get_description (void);
-GList             *plugin_get_connection_params (void);
-gchar             *plugin_get_dsn_spec (void);
+const gchar *plugin_get_name (void);
+const gchar *plugin_get_description (void);
+GList *plugin_get_connection_params (void);
 GdaServerProvider *plugin_create_provider (void);
 
 const gchar *
@@ -56,19 +54,6 @@ plugin_get_connection_params (void)
 	return list;
 }
 
-gchar *
-plugin_get_dsn_spec (void)
-{
-	gchar *specs, *file;
-	gsize len;
-
-	file = g_build_filename (LIBGDA_DATA_DIR, "sqlite_specs_dsn.xml", NULL);
-	if (g_file_get_contents (file, &specs, &len, NULL)) 
-		return specs;
-	else
-		return NULL;
-}
-
 GdaServerProvider *
 plugin_create_provider (void)
 {
