@@ -1,10 +1,9 @@
 /* GDA Sybase provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998-2002 The GNOME Foundation.
  *
  * AUTHORS:
  *         Mike Wingert <wingert.3@postbox.acs.ohio-state.edu>
  *         Holger Thon <holger.thon@gnome-db.org>
- *         Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -26,7 +25,7 @@
 #  include <config.h>
 #endif
 
-#include <glib/gi18n-lib.h>
+#include <libgda/gda-intl.h>
 #include <libgda/gda-data-model.h>
 #include <string.h>
 #include <ctpublic.h>
@@ -39,7 +38,7 @@
 #  undef PARENT_TYPE
 #endif
 
-#define PARENT_TYPE GDA_TYPE_DATA_MODEL_ROW
+#define PARENT_TYPE GDA_TYPE_DATA_MODEL
 
 static GObjectClass *parent_class = NULL;
 
@@ -48,57 +47,68 @@ static void gda_sybase_recordset_init (GdaSybaseRecordset *recset,
                                        GdaSybaseRecordsetClass *klass);
 static void gda_sybase_recordset_finalize (GObject *object);
 
-static void gda_sybase_recordset_describe_column (GdaDataModel *model, gint col);
-static gint gda_sybase_recordset_get_n_rows (GdaDataModelRow *model);
-static gint gda_sybase_recordset_get_n_columns (GdaDataModelRow *model);
-static const GdaRow *gda_sybase_recordset_get_row (GdaDataModelRow *model,
-                                                   gint row, GError **error);
-static const GdaValue *gda_sybase_recordset_get_value_at (GdaDataModelRow *model,
+static GdaFieldAttributes *gda_sybase_recordset_describe_column (GdaDataModel *model,
+                                                                 gint col);
+static gint gda_sybase_recordset_get_n_rows (GdaDataModel *model);
+static gint gda_sybase_recordset_get_n_columns (GdaDataModel *model);
+static const GdaRow *gda_sybase_recordset_get_row (GdaDataModel *model,
+                                                   gint row);
+static const GdaValue *gda_sybase_recordset_get_value_at (GdaDataModel *model,
                                                           gint col,
                                                           gint row);
 
-static void
+static GdaFieldAttributes *
 gda_sybase_recordset_describe_column (GdaDataModel *model, gint col)
 {
 	GdaSybaseRecordset *recset = (GdaSybaseRecordset *) model;
 	CS_DATAFMT         *colinfo = NULL;
-	GdaColumn          *attribs = NULL;
+	GdaFieldAttributes *attribs = NULL;
 	gchar              name[256];
 
-	g_return_if_fail (GDA_IS_SYBASE_RECORDSET (recset));
-	g_return_if_fail (recset->priv != NULL);
-	g_return_if_fail (recset->priv->columns != NULL);
-	g_return_if_fail (col < recset->priv->columns->len);
+	g_return_val_if_fail (GDA_IS_SYBASE_RECORDSET (recset), NULL);
+	g_return_val_if_fail (recset->priv != NULL, NULL);
+	g_return_val_if_fail (recset->priv->columns != NULL, NULL);
 
+	if (col >= recset->priv->columns->len) {
+		return NULL;
+	}
 	colinfo = g_ptr_array_index (recset->priv->columns, col);
-	g_return_if_fail (colinfo);
 
-	attribs = gda_data_model_describe_column (model, col);
+	if (!colinfo) {
+		return NULL;
+	}
+
+	attribs = gda_field_attributes_new ();
+
+	if (!attribs) {
+		return NULL;
+	}
 
 	memcpy (name, colinfo->name,
 	        colinfo->namelen);
-	name[colinfo->namelen + 1] = 0
+	//name[colinfo->namelen + 1] = '\0';
 
-		gda_column_set_title (attribs, name);
-	gda_column_set_name (attribs, name);
-	gda_column_set_scale (attribs, colinfo->scale);
-	gda_column_set_gda_type (attribs,
-				gda_sybase_get_value_type (colinfo->datatype));
-	gda_column_set_defined_size (attribs, colinfo->maxlength);
+	gda_field_attributes_set_name (attribs, name);
+	gda_field_attributes_set_scale (attribs, colinfo->scale);
+	gda_field_attributes_set_gdatype (attribs,
+	         gda_sybase_get_value_type (colinfo->datatype));
+	gda_field_attributes_set_defined_size (attribs, colinfo->maxlength);
 
 	// FIXME:
-	gda_column_set_references (attribs, "");
-	gda_column_set_primary_key (attribs, FALSE);
-	//gda_column_set_primary_key (attribs,
+	gda_field_attributes_set_references (attribs, "");
+	gda_field_attributes_set_primary_key (attribs, FALSE);
+	//gda_field_attributes_set_primary_key (attribs,
 	//         (colinfo->status & CS_KEY) == CS_KEY);
-	gda_column_set_unique_key (attribs, FALSE);
+	gda_field_attributes_set_unique_key (attribs, FALSE);
 	
-	gda_column_set_allow_null (attribs,
-				   (colinfo->status & CS_CANBENULL) == CS_CANBENULL);
+	gda_field_attributes_set_allow_null (attribs,
+	         (colinfo->status & CS_CANBENULL) == CS_CANBENULL);
+	
+	return attribs;
 }
 
 static gint
-gda_sybase_recordset_get_n_rows (GdaDataModelRow *model)
+gda_sybase_recordset_get_n_rows (GdaDataModel *model)
 {
 	GdaSybaseRecordset *recset = (GdaSybaseRecordset *) model;
 
@@ -107,7 +117,7 @@ gda_sybase_recordset_get_n_rows (GdaDataModelRow *model)
 }
 
 static gint
-gda_sybase_recordset_get_n_columns (GdaDataModelRow *model)
+gda_sybase_recordset_get_n_columns (GdaDataModel *model)
 {
 	GdaSybaseRecordset *recset = (GdaSybaseRecordset *) model;
 
@@ -116,7 +126,7 @@ gda_sybase_recordset_get_n_columns (GdaDataModelRow *model)
 }
 
 static const GdaRow *
-gda_sybase_recordset_get_row (GdaDataModelRow *model, gint row, GError **error)
+gda_sybase_recordset_get_row (GdaDataModel *model, gint row)
 {
 	GdaSybaseRecordset *recset = (GdaSybaseRecordset *) model;
 
@@ -133,7 +143,7 @@ gda_sybase_recordset_get_row (GdaDataModelRow *model, gint row, GError **error)
 }
 
 static const GdaValue *
-gda_sybase_recordset_get_value_at (GdaDataModelRow *model, gint col, gint row)
+gda_sybase_recordset_get_value_at (GdaDataModel *model, gint col, gint row)
 {
 	GdaSybaseRecordset *recset = (GdaSybaseRecordset *) model;
 	const GdaRow *fields;
@@ -144,7 +154,7 @@ gda_sybase_recordset_get_value_at (GdaDataModelRow *model, gint col, gint row)
 	if (col >= recset->priv->colcnt)
 		return NULL;
 
-	fields = gda_sybase_recordset_get_row (model, row, NULL);
+	fields = gda_sybase_recordset_get_row (model, row);
 	return fields != NULL ? gda_row_get_value ((GdaRow *)fields, col) : NULL;
 }
 
@@ -152,11 +162,12 @@ static void
 gda_sybase_recordset_class_init (GdaSybaseRecordsetClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelRowClass *model_class = GDA_DATA_MODEL_ROW_CLASS (klass);
+	GdaDataModelClass *model_class = GDA_DATA_MODEL_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_sybase_recordset_finalize;
+	model_class->describe_column = gda_sybase_recordset_describe_column;
 	model_class->get_n_rows = gda_sybase_recordset_get_n_rows;
 	model_class->get_n_columns = gda_sybase_recordset_get_n_columns;
 	model_class->get_row = gda_sybase_recordset_get_row;
@@ -188,7 +199,7 @@ gda_sybase_recordset_finalize (GObject *object)
 				GdaRow *row = (GdaRow *) g_ptr_array_index (recset->priv->rows, 0);
 
 				if (row != NULL) {
-					g_object_unref (row);
+					gda_row_free (row);
 					row = NULL;
 				}
 				g_ptr_array_remove_index (recset->priv->rows, 0);
@@ -278,7 +289,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 	GdaSybaseRecordset *srecset = NULL;
 	GdaSybaseField     *sfield = NULL;
 	GdaRow             *row = NULL;
-	GdaConnectionEvent *error = NULL;
+	GdaError           *error = NULL;
 	gboolean           columns_set = FALSE;
 	gboolean           sfields_allocated = TRUE;
 	gint               i = 0;
@@ -296,7 +307,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		srecset->priv->scnc = scnc;
 	} else {
 		// most probably a lack of ram failure, 
-		// so don't alloc e.g. an instance of GdaConnectionEvent
+		// so don't alloc e.g. an instance of GdaError
 		if (srecset) {
 			g_object_unref (srecset);
 			srecset = NULL;
@@ -319,7 +330,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		error = gda_sybase_make_error (scnc,
 		                               _("%s failed while processing a row result."),
 		                               "ct_res_info()");
-		gda_connection_add_event (cnc, error);
+		gda_connection_add_error (cnc, error);
 		sybase_check_messages(cnc);
 		return NULL;
 	}
@@ -327,7 +338,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		error = gda_sybase_make_error (scnc,
 		                               _("%s returned <= 0 columns."),
 		                               "ct_res_info()");
-		gda_connection_add_event (cnc, error);
+		gda_connection_add_error (cnc, error);
 		return NULL;
 	}
 
@@ -369,7 +380,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		if (scnc->ret != CS_SUCCEED) {
 			error = gda_sybase_make_error (scnc, 
 			                               _("Could not run %s on column %d"), "ct_describe()", i);
-			gda_connection_add_event (cnc, error);
+			gda_connection_add_error (cnc, error);
 			sybase_check_messages(cnc);
 			break;
 		}
@@ -378,7 +389,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		if (sfield->data == NULL) {
 			error = gda_sybase_make_error (scnc,
 			                               _("Could not allocate data placeholder for column %d"), i);
-			gda_connection_add_event (cnc, error);
+			gda_connection_add_error (cnc, error);
 			sybase_check_messages(cnc);
 			break;
 		}
@@ -390,13 +401,15 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		if (scnc->ret != CS_SUCCEED) {
 			error = gda_sybase_make_error (scnc,
 			                               _("Could not run %s on column %d"), "ct_bind()", i);
-			gda_connection_add_event (cnc, error);
+			gda_connection_add_error (cnc, error);
 			sybase_check_messages(cnc);
 			break;
 		}
 
-		if (sfield->fmt.namelen > 0) 
-			gda_sybase_recordset_describe_column (GDA_DATA_MODEL (srecset), i);
+		if (sfield->fmt.namelen > 0) {
+			gda_data_model_set_column_title (GDA_DATA_MODEL (srecset),
+			                                 i, sfield->fmt.name);
+		}
 	}
 
 	// catch error: ct_describe or g_new0 or ct_bind failed
@@ -407,7 +420,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		scnc->ret = ct_cancel (NULL, scnc->cmd, CS_CANCEL_ALL);
 		if (scnc->ret != CS_SUCCEED) {
 			error = gda_sybase_make_error (scnc, _("Could not run %s to cancel processing row resultset."), "ct_cancel");
-			gda_connection_add_event (cnc, error);
+			gda_connection_add_error (cnc, error);
 			sybase_check_messages(cnc);
 		}
 		
@@ -423,7 +436,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		
 		if (scnc->ret == CS_ROW_FAIL) {
 			error = gda_sybase_make_error (scnc, _("%s failed on row %d"), "ct_fetch()", row_cnt);
-			gda_connection_add_event (cnc, error);
+			gda_connection_add_error (cnc, error);
 			sybase_check_messages(cnc);
 		}
 		
@@ -439,19 +452,19 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 //	sybase_debug_msg (_("Got %d of %d rows"), srecset->priv->rowcnt, row_cnt);
 	// step 5: verify ct_fetch's last return code
 	switch (scnc->ret) {
-	case CS_END_DATA:
+		case CS_END_DATA:
 //			sybase_debug_msg (_("Row processing succeeded."));
-		break;
-	case CS_CANCELED:
+			break;
+		case CS_CANCELED:
 //			sybase_debug_msg (_("Row processing canceled."));
-		break;
-	case CS_FAIL:
+			break;
+		case CS_FAIL:
 //			sybase_debug_msg (_("%s returned CS_FAIL. Probably o.k."), "ct_fetch()");
-		break;
-	default:
-		error = gda_sybase_make_error (scnc, _("%s terminated with unexpected return code."), "ct_fetch()");
-		gda_connection_add_event (cnc, error);
-		sybase_check_messages(cnc);
+			break;
+		default:
+			error = gda_sybase_make_error (scnc, _("%s terminated with unexpected return code."), "ct_fetch()");
+			gda_connection_add_error (cnc, error);
+			sybase_check_messages(cnc);
 	}
 	
 	return srecset;
@@ -459,8 +472,8 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 
 
 GdaSybaseRecordset *
-gda_sybase_process_msg_result (GdaConnection *cnc,
-			       GdaSybaseConnectionData *scnc)
+gda_sybase_process_msg_result(GdaConnection *cnc,
+															GdaSybaseConnectionData *scnc)
 {
 	
 	// in ASE when a stored procedure uses the print statement, it 
@@ -473,7 +486,7 @@ gda_sybase_process_msg_result (GdaConnection *cnc,
 	GdaValue *val = NULL;
 	gchar *message;
 	CS_INT msgcnt = 0;
-	GdaConnectionEvent *error;
+	GdaError *error;
 	CS_SERVERMSG msg; 
 	CS_RETCODE ret;
 
@@ -485,43 +498,41 @@ gda_sybase_process_msg_result (GdaConnection *cnc,
 
 	// see if there is a message
 	ret = ct_diag (scnc->connection,
-		       CS_STATUS, 
-		       CS_ALLMSG_TYPE, 
-		       CS_UNUSED,
-		       &msgcnt);
+								 CS_STATUS, 
+								 CS_ALLMSG_TYPE, 
+								 CS_UNUSED,
+								 &msgcnt);
 	
 	if ( ret != CS_SUCCEED ) {
-		error = gda_connection_event_new();
+		error = gda_error_new();
 		g_return_val_if_fail (error != NULL, FALSE);
-		gda_connection_event_set_description (error, 
-						      _("An error occurred when attempting to retrieve a server message count for resultset"));
-		gda_connection_event_set_code (error, -1);
-		gda_connection_event_set_source (error, "gda-sybase");
-		gda_connection_event_set_sqlstate (error, _("Not available"));					
-		gda_connection_add_event (cnc, error);		
+		gda_error_set_description (error, _("an error occured when calling ct_diag attempting to retrieve a server message count for resultset"));
+		gda_error_set_number (error, -1);
+		gda_error_set_source (error, "gda-sybase");
+		gda_error_set_sqlstate (error, _("Not available"));					
+		gda_connection_add_error (cnc, error);		
 		return NULL;
 	}
 
 	if ( msgcnt < 1 ){
-		sybase_debug_msg (_("attempting to make recordset and msg count != 1 !"));
-		return NULL;
+			sybase_debug_msg (_("attempting to make recordset and msg count != 1 !"));
+			return NULL;
 	}
 
 	ret = ct_diag (scnc->connection, 
-		       CS_GET, 
-		       CS_SERVERMSG_TYPE, 
-		       1, 
-		       &msg); 
+																CS_GET, 
+																CS_SERVERMSG_TYPE, 
+																1, 
+																&msg); 
 	if ( ret != CS_SUCCEED ) { 
-		error = gda_connection_event_new();
+		error = gda_error_new();
 		g_return_val_if_fail (error != NULL, FALSE);
-
-		gda_connection_event_set_description (error,
-						      _("An error occurred when attempting to retrieve a server message for recordset"));
-		gda_connection_event_set_code (error, -1);
-		gda_connection_event_set_source (error, "gda-sybase");
-		gda_connection_event_set_sqlstate (error, _("Not available"));					
-		gda_connection_add_event (cnc, error);
+		gda_error_set_description (error,
+															 _("an error occured when calling ct_diag attempting to retrieve a server message for recordset"));
+		gda_error_set_number (error, -1);
+		gda_error_set_source (error, "gda-sybase");
+		gda_error_set_sqlstate (error, _("Not available"));					
+		gda_connection_add_error (cnc, error);
 		return NULL;					
 	} 	
 
@@ -558,30 +569,31 @@ gda_sybase_process_msg_result (GdaConnection *cnc,
 	row = gda_row_new(GDA_DATA_MODEL(srecset), 1);
 	val = gda_row_get_value(row,0);
 
-	message = g_strdup_printf("%s", (msg.text) ? msg.text : "");
+	message = g_strdup_printf("%s",
+																											(msg.text) ? msg.text : "");
+
 	sfield->fmt.maxlength = strlen(message);
 
 	srecset->priv->rowcnt = 1;	
-	gda_value_set_string(val,(gchar *)message);
+ gda_value_set_string(val,(gchar *)message);
 	g_ptr_array_add(srecset->priv->rows, row);
 
 	// clear the message so we don't get bugged by it later
 	ret = ct_diag (scnc->connection, 
-		       CS_CLEAR, 
-		       CS_SERVERMSG_TYPE, 
-		       CS_UNUSED, 
-		       NULL);   
+																CS_CLEAR, 
+																CS_SERVERMSG_TYPE, 
+																CS_UNUSED, 
+																NULL);   
 	if ( ret != CS_SUCCEED ) { 
-		error = gda_connection_event_new();
-		g_return_val_if_fail (error != NULL, FALSE);
-		gda_connection_event_set_description (error, 
-						      _("call to ct_diag failed when attempting to clear the server messages"));
-		gda_connection_event_set_code (error, -1);
-		gda_connection_event_set_source (error, "gda-sybase");
-		gda_connection_event_set_sqlstate (error, _("Not available"));					
-		gda_connection_add_event (cnc, error);
-		return NULL;
-	} 	
+			error = gda_error_new();
+			g_return_val_if_fail (error != NULL, FALSE);
+			gda_error_set_description (error, _("call to ct_diag failed when attempting to clear the server messages"));
+			gda_error_set_number (error, -1);
+			gda_error_set_source (error, "gda-sybase");
+			gda_error_set_sqlstate (error, _("Not available"));					
+			gda_connection_add_error (cnc, error);
+			return NULL;
+		} 	
 
 	return srecset;
 }
